=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2011-03-09 01:31:04 +0000
+++ bzrlib/bzrdir.py	2011-03-10 13:29:54 +0000
@@ -72,6 +72,7 @@
 
 from bzrlib import (
     hooks,
+    registry,
     )
 from bzrlib.symbol_versioning import (
     deprecated_in,
@@ -1203,16 +1204,18 @@
 class BzrProber(controldir.Prober):
     """Prober for formats that use a .bzr/ control directory."""
 
-    _formats = {}
+    formats = registry.FormatRegistry(controldir.network_format_registry)
     """The known .bzr formats."""
 
     @classmethod
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def register_bzrdir_format(klass, format):
-        klass._formats[format.get_format_string()] = format
+        klass.formats.register(format.get_format_string(), format)
 
     @classmethod
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def unregister_bzrdir_format(klass, format):
-        del klass._formats[format.get_format_string()]
+        klass.formats.remove(format.get_format_string())
 
     @classmethod
     def probe_transport(klass, transport):
@@ -1222,7 +1225,7 @@
         except errors.NoSuchFile:
             raise errors.NotBranchError(path=transport.base)
         try:
-            return klass._formats[format_string]
+            return klass.formats.get(format_string)
         except KeyError:
             raise errors.UnknownFormatError(format=format_string, kind='bzrdir')
 
@@ -1458,11 +1461,24 @@
 
     @classmethod
     def register_format(klass, format):
-        BzrProber.register_bzrdir_format(format)
-        # bzr native formats have a network name of their format string.
-        controldir.network_format_registry.register(format.get_format_string(), format.__class__)
+        """Register a new BzrDir format.
+        """
+        BzrProber.formats.register(format.get_format_string(), format)
         controldir.ControlDirFormat.register_format(format)
 
+    @classmethod
+    def register_lazy_format(klass, format_string, module_name, member_name):
+        """Lazily register a new BzrDir format.
+
+        :param format_string: Format string
+        :param module_name: Name of module with the BzrDirFormat subclass
+        :param member_name: Class name of the BzrDirFormat
+        """
+        BzrProber.formats.register_lazy(format_string, module_name,
+            member_name)
+        controldir.ControlDirFormat.register_lazy_format(
+            module_name, member_name)
+
     def _supply_sub_formats_to(self, other_format):
         """Give other_format the same values for sub formats as this has.
 
@@ -1477,9 +1493,14 @@
 
     @classmethod
     def unregister_format(klass, format):
-        BzrProber.unregister_bzrdir_format(format)
+        BzrProber.formats.remove(format.get_format_string())
         controldir.ControlDirFormat.unregister_format(format)
-        controldir.network_format_registry.remove(format.get_format_string())
+
+    @classmethod
+    def unregister_lazy_format(klass, format_string, module_name, member_name):
+        BzrProber.formats.remove(format_string)
+        controldir.ControlDirFormat.unregister_lazy_format(
+            module_name, member_name)
 
 
 class BzrDirMetaFormat1(BzrDirFormat):

=== modified file 'bzrlib/controldir.py'
--- bzrlib/controldir.py	2011-03-05 02:54:47 +0000
+++ bzrlib/controldir.py	2011-03-10 13:00:05 +0000
@@ -713,7 +713,7 @@
     _formats = []
     """The registered control formats - .bzr, ....
 
-    This is a list of ControlDirFormat objects.
+    This is a list of _ObjectGetters of ControlDirFormat objects.
     """
 
     _server_probers = []
@@ -773,10 +773,18 @@
 
     @classmethod
     def register_format(klass, format):
-        """Register a format that does not use '.bzr' for its control dir.
-
-        """
-        klass._formats.append(format)
+        """Register a control dir format.
+
+        """
+        klass._formats.append(registry._ObjectGetter(format))
+
+    @classmethod
+    def register_lazy_format(klass, module_name, member_name):
+        """Lazily register a control dir format.
+
+        """
+        klass._formats.append(registry._LazyObjectGetter(
+            module_name, member_name))
 
     @classmethod
     def register_prober(klass, prober):
@@ -809,13 +817,18 @@
 
     @classmethod
     def unregister_format(klass, format):
-        klass._formats.remove(format)
+        klass._formats.remove(registry._ObjectGetter(format))
+
+    @classmethod
+    def unregister_lazy_format(klass, module_name, member_name):
+        klass._formats.remove(registry._LazyObjectGetter(
+            module_name, member_name))
 
     @classmethod
     def known_formats(klass):
         """Return all the known formats.
         """
-        return set(klass._formats)
+        return set([objgetter.get_obj() for objgetter in klass._formats])
 
     @classmethod
     def find_format(klass, transport, _server_formats=True):

=== modified file 'bzrlib/plugins/weave_fmt/__init__.py'
--- bzrlib/plugins/weave_fmt/__init__.py	2011-03-07 17:10:54 +0000
+++ bzrlib/plugins/weave_fmt/__init__.py	2011-03-10 13:29:54 +0000
@@ -84,10 +84,16 @@
     deprecated=True)
 
 
-from bzrlib.plugins.weave_fmt.bzrdir import BzrDirFormat4, BzrDirFormat5, BzrDirFormat6
-bzrdir.BzrDirFormat.register_format(BzrDirFormat4())
-bzrdir.BzrDirFormat.register_format(BzrDirFormat5())
-bzrdir.BzrDirFormat.register_format(BzrDirFormat6())
+bzrdir.BzrDirFormat.register_lazy_format(
+    "Bazaar-NG branch, format 0.0.4\n", "bzrlib.plugins.weave_fmt.bzrdir",
+    "BzrDirFormat4")
+bzrdir.BzrDirFormat.register_lazy_format(
+    "Bazaar-NG branch, format 5\n", "bzrlib.plugins.weave_fmt.bzrdir",
+    "BzrDirFormat5")
+bzrdir.BzrDirFormat.register_lazy_format(
+    "Bazaar-NG branch, format 6\n", "bzrlib.plugins.weave_fmt.bzrdir",
+    "BzrDirFormat6")
+
 
 branch.format_registry.register_extra_lazy(
     'bzrlib.plugins.weave_fmt.branch', 'BzrBranchFormat4')

=== removed file 'bzrlib/plugins/weave_fmt/bzrdir.py'
--- bzrlib/plugins/weave_fmt/bzrdir.py	2011-03-07 01:58:14 +0000
+++ bzrlib/plugins/weave_fmt/bzrdir.py	1970-01-01 00:00:00 +0000
@@ -1,969 +0,0 @@
-# Copyright (C) 2006-2010 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Weave-era BzrDir formats."""
-
-from bzrlib.bzrdir import (
-    BzrDir,
-    BzrDirFormat,
-    BzrDirMetaFormat1,
-    )
-from bzrlib.controldir import (
-    Converter,
-    format_registry,
-    )
-from bzrlib.lazy_import import lazy_import
-lazy_import(globals(), """
-import os
-import warnings
-
-from bzrlib import (
-    errors,
-    graph,
-    lockable_files,
-    lockdir,
-    osutils,
-    revision as _mod_revision,
-    trace,
-    ui,
-    urlutils,
-    versionedfile,
-    weave,
-    xml5,
-    )
-from bzrlib.store.versioned import VersionedFileStore
-from bzrlib.transactions import WriteTransaction
-from bzrlib.transport import (
-    get_transport,
-    local,
-    )
-from bzrlib.plugins.weave_fmt import xml4
-""")
-
-
-class BzrDirFormatAllInOne(BzrDirFormat):
-    """Common class for formats before meta-dirs."""
-
-    fixed_components = True
-
-    def initialize_on_transport_ex(self, transport, use_existing_dir=False,
-        create_prefix=False, force_new_repo=False, stacked_on=None,
-        stack_on_pwd=None, repo_format_name=None, make_working_trees=None,
-        shared_repo=False):
-        """See BzrDirFormat.initialize_on_transport_ex."""
-        require_stacking = (stacked_on is not None)
-        # Format 5 cannot stack, but we've been asked to - actually init
-        # a Meta1Dir
-        if require_stacking:
-            format = BzrDirMetaFormat1()
-            return format.initialize_on_transport_ex(transport,
-                use_existing_dir=use_existing_dir, create_prefix=create_prefix,
-                force_new_repo=force_new_repo, stacked_on=stacked_on,
-                stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
-                make_working_trees=make_working_trees, shared_repo=shared_repo)
-        return BzrDirFormat.initialize_on_transport_ex(self, transport,
-            use_existing_dir=use_existing_dir, create_prefix=create_prefix,
-            force_new_repo=force_new_repo, stacked_on=stacked_on,
-            stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
-            make_working_trees=make_working_trees, shared_repo=shared_repo)
-
-
-class BzrDirFormat5(BzrDirFormatAllInOne):
-    """Bzr control format 5.
-
-    This format is a combined format for working tree, branch and repository.
-    It has:
-     - Format 2 working trees [always]
-     - Format 4 branches [always]
-     - Format 5 repositories [always]
-       Unhashed stores in the repository.
-    """
-
-    _lock_class = lockable_files.TransportLock
-
-    def get_format_string(self):
-        """See BzrDirFormat.get_format_string()."""
-        return "Bazaar-NG branch, format 5\n"
-
-    def get_branch_format(self):
-        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
-        return BzrBranchFormat4()
-
-    def get_format_description(self):
-        """See BzrDirFormat.get_format_description()."""
-        return "All-in-one format 5"
-
-    def get_converter(self, format=None):
-        """See BzrDirFormat.get_converter()."""
-        # there is one and only one upgrade path here.
-        return ConvertBzrDir5To6()
-
-    def _initialize_for_clone(self, url):
-        return self.initialize_on_transport(get_transport(url), _cloning=True)
-
-    def initialize_on_transport(self, transport, _cloning=False):
-        """Format 5 dirs always have working tree, branch and repository.
-
-        Except when they are being cloned.
-        """
-        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
-        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat5
-        result = (super(BzrDirFormat5, self).initialize_on_transport(transport))
-        RepositoryFormat5().initialize(result, _internal=True)
-        if not _cloning:
-            branch = BzrBranchFormat4().initialize(result)
-            result._init_workingtree()
-        return result
-
-    def network_name(self):
-        return self.get_format_string()
-
-    def _open(self, transport):
-        """See BzrDirFormat._open."""
-        return BzrDir5(transport, self)
-
-    def __return_repository_format(self):
-        """Circular import protection."""
-        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat5
-        return RepositoryFormat5()
-    repository_format = property(__return_repository_format)
-
-
-class BzrDirFormat6(BzrDirFormatAllInOne):
-    """Bzr control format 6.
-
-    This format is a combined format for working tree, branch and repository.
-    It has:
-     - Format 2 working trees [always]
-     - Format 4 branches [always]
-     - Format 6 repositories [always]
-    """
-
-    _lock_class = lockable_files.TransportLock
-
-    def get_format_string(self):
-        """See BzrDirFormat.get_format_string()."""
-        return "Bazaar-NG branch, format 6\n"
-
-    def get_format_description(self):
-        """See BzrDirFormat.get_format_description()."""
-        return "All-in-one format 6"
-
-    def get_branch_format(self):
-        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
-        return BzrBranchFormat4()
-
-    def get_converter(self, format=None):
-        """See BzrDirFormat.get_converter()."""
-        # there is one and only one upgrade path here.
-        return ConvertBzrDir6ToMeta()
-
-    def _initialize_for_clone(self, url):
-        return self.initialize_on_transport(get_transport(url), _cloning=True)
-
-    def initialize_on_transport(self, transport, _cloning=False):
-        """Format 6 dirs always have working tree, branch and repository.
-
-        Except when they are being cloned.
-        """
-        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
-        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat6
-        result = super(BzrDirFormat6, self).initialize_on_transport(transport)
-        RepositoryFormat6().initialize(result, _internal=True)
-        if not _cloning:
-            branch = BzrBranchFormat4().initialize(result)
-            result._init_workingtree()
-        return result
-
-    def network_name(self):
-        return self.get_format_string()
-
-    def _open(self, transport):
-        """See BzrDirFormat._open."""
-        return BzrDir6(transport, self)
-
-    def __return_repository_format(self):
-        """Circular import protection."""
-        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat6
-        return RepositoryFormat6()
-    repository_format = property(__return_repository_format)
-
-
-class ConvertBzrDir4To5(Converter):
-    """Converts format 4 bzr dirs to format 5."""
-
-    def __init__(self):
-        super(ConvertBzrDir4To5, self).__init__()
-        self.converted_revs = set()
-        self.absent_revisions = set()
-        self.text_count = 0
-        self.revisions = {}
-
-    def convert(self, to_convert, pb):
-        """See Converter.convert()."""
-        self.bzrdir = to_convert
-        if pb is not None:
-            warnings.warn("pb parameter to convert() is deprecated")
-        self.pb = ui.ui_factory.nested_progress_bar()
-        try:
-            ui.ui_factory.note('starting upgrade from format 4 to 5')
-            if isinstance(self.bzrdir.transport, local.LocalTransport):
-                self.bzrdir.get_workingtree_transport(None).delete('stat-cache')
-            self._convert_to_weaves()
-            return BzrDir.open(self.bzrdir.user_url)
-        finally:
-            self.pb.finished()
-
-    def _convert_to_weaves(self):
-        ui.ui_factory.note('note: upgrade may be faster if all store files are ungzipped first')
-        try:
-            # TODO permissions
-            stat = self.bzrdir.transport.stat('weaves')
-            if not S_ISDIR(stat.st_mode):
-                self.bzrdir.transport.delete('weaves')
-                self.bzrdir.transport.mkdir('weaves')
-        except errors.NoSuchFile:
-            self.bzrdir.transport.mkdir('weaves')
-        # deliberately not a WeaveFile as we want to build it up slowly.
-        self.inv_weave = weave.Weave('inventory')
-        # holds in-memory weaves for all files
-        self.text_weaves = {}
-        self.bzrdir.transport.delete('branch-format')
-        self.branch = self.bzrdir.open_branch()
-        self._convert_working_inv()
-        rev_history = self.branch.revision_history()
-        # to_read is a stack holding the revisions we still need to process;
-        # appending to it adds new highest-priority revisions
-        self.known_revisions = set(rev_history)
-        self.to_read = rev_history[-1:]
-        while self.to_read:
-            rev_id = self.to_read.pop()
-            if (rev_id not in self.revisions
-                and rev_id not in self.absent_revisions):
-                self._load_one_rev(rev_id)
-        self.pb.clear()
-        to_import = self._make_order()
-        for i, rev_id in enumerate(to_import):
-            self.pb.update('converting revision', i, len(to_import))
-            self._convert_one_rev(rev_id)
-        self.pb.clear()
-        self._write_all_weaves()
-        self._write_all_revs()
-        ui.ui_factory.note('upgraded to weaves:')
-        ui.ui_factory.note('  %6d revisions and inventories' % len(self.revisions))
-        ui.ui_factory.note('  %6d revisions not present' % len(self.absent_revisions))
-        ui.ui_factory.note('  %6d texts' % self.text_count)
-        self._cleanup_spare_files_after_format4()
-        self.branch._transport.put_bytes(
-            'branch-format',
-            BzrDirFormat5().get_format_string(),
-            mode=self.bzrdir._get_file_mode())
-
-    def _cleanup_spare_files_after_format4(self):
-        # FIXME working tree upgrade foo.
-        for n in 'merged-patches', 'pending-merged-patches':
-            try:
-                ## assert os.path.getsize(p) == 0
-                self.bzrdir.transport.delete(n)
-            except errors.NoSuchFile:
-                pass
-        self.bzrdir.transport.delete_tree('inventory-store')
-        self.bzrdir.transport.delete_tree('text-store')
-
-    def _convert_working_inv(self):
-        inv = xml4.serializer_v4.read_inventory(
-                self.branch._transport.get('inventory'))
-        new_inv_xml = xml5.serializer_v5.write_inventory_to_string(inv, working=True)
-        self.branch._transport.put_bytes('inventory', new_inv_xml,
-            mode=self.bzrdir._get_file_mode())
-
-    def _write_all_weaves(self):
-        controlweaves = VersionedFileStore(self.bzrdir.transport, prefixed=False,
-            versionedfile_class=weave.WeaveFile)
-        weave_transport = self.bzrdir.transport.clone('weaves')
-        weaves = VersionedFileStore(weave_transport, prefixed=False,
-                versionedfile_class=weave.WeaveFile)
-        transaction = WriteTransaction()
-
-        try:
-            i = 0
-            for file_id, file_weave in self.text_weaves.items():
-                self.pb.update('writing weave', i, len(self.text_weaves))
-                weaves._put_weave(file_id, file_weave, transaction)
-                i += 1
-            self.pb.update('inventory', 0, 1)
-            controlweaves._put_weave('inventory', self.inv_weave, transaction)
-            self.pb.update('inventory', 1, 1)
-        finally:
-            self.pb.clear()
-
-    def _write_all_revs(self):
-        """Write all revisions out in new form."""
-        self.bzrdir.transport.delete_tree('revision-store')
-        self.bzrdir.transport.mkdir('revision-store')
-        revision_transport = self.bzrdir.transport.clone('revision-store')
-        # TODO permissions
-        from bzrlib.xml5 import serializer_v5
-        from bzrlib.plugins.weave_fmt.repository import RevisionTextStore
-        revision_store = RevisionTextStore(revision_transport,
-            serializer_v5, False, versionedfile.PrefixMapper(),
-            lambda:True, lambda:True)
-        try:
-            for i, rev_id in enumerate(self.converted_revs):
-                self.pb.update('write revision', i, len(self.converted_revs))
-                text = serializer_v5.write_revision_to_string(
-                    self.revisions[rev_id])
-                key = (rev_id,)
-                revision_store.add_lines(key, None, osutils.split_lines(text))
-        finally:
-            self.pb.clear()
-
-    def _load_one_rev(self, rev_id):
-        """Load a revision object into memory.
-
-        Any parents not either loaded or abandoned get queued to be
-        loaded."""
-        self.pb.update('loading revision',
-                       len(self.revisions),
-                       len(self.known_revisions))
-        if not self.branch.repository.has_revision(rev_id):
-            self.pb.clear()
-            ui.ui_factory.note('revision {%s} not present in branch; '
-                         'will be converted as a ghost' %
-                         rev_id)
-            self.absent_revisions.add(rev_id)
-        else:
-            rev = self.branch.repository.get_revision(rev_id)
-            for parent_id in rev.parent_ids:
-                self.known_revisions.add(parent_id)
-                self.to_read.append(parent_id)
-            self.revisions[rev_id] = rev
-
-    def _load_old_inventory(self, rev_id):
-        f = self.branch.repository.inventory_store.get(rev_id)
-        try:
-            old_inv_xml = f.read()
-        finally:
-            f.close()
-        inv = xml4.serializer_v4.read_inventory_from_string(old_inv_xml)
-        inv.revision_id = rev_id
-        rev = self.revisions[rev_id]
-        return inv
-
-    def _load_updated_inventory(self, rev_id):
-        inv_xml = self.inv_weave.get_text(rev_id)
-        inv = xml5.serializer_v5.read_inventory_from_string(inv_xml, rev_id)
-        return inv
-
-    def _convert_one_rev(self, rev_id):
-        """Convert revision and all referenced objects to new format."""
-        rev = self.revisions[rev_id]
-        inv = self._load_old_inventory(rev_id)
-        present_parents = [p for p in rev.parent_ids
-                           if p not in self.absent_revisions]
-        self._convert_revision_contents(rev, inv, present_parents)
-        self._store_new_inv(rev, inv, present_parents)
-        self.converted_revs.add(rev_id)
-
-    def _store_new_inv(self, rev, inv, present_parents):
-        new_inv_xml = xml5.serializer_v5.write_inventory_to_string(inv)
-        new_inv_sha1 = osutils.sha_string(new_inv_xml)
-        self.inv_weave.add_lines(rev.revision_id,
-                                 present_parents,
-                                 new_inv_xml.splitlines(True))
-        rev.inventory_sha1 = new_inv_sha1
-
-    def _convert_revision_contents(self, rev, inv, present_parents):
-        """Convert all the files within a revision.
-
-        Also upgrade the inventory to refer to the text revision ids."""
-        rev_id = rev.revision_id
-        trace.mutter('converting texts of revision {%s}', rev_id)
-        parent_invs = map(self._load_updated_inventory, present_parents)
-        entries = inv.iter_entries()
-        entries.next()
-        for path, ie in entries:
-            self._convert_file_version(rev, ie, parent_invs)
-
-    def _convert_file_version(self, rev, ie, parent_invs):
-        """Convert one version of one file.
-
-        The file needs to be added into the weave if it is a merge
-        of >=2 parents or if it's changed from its parent.
-        """
-        file_id = ie.file_id
-        rev_id = rev.revision_id
-        w = self.text_weaves.get(file_id)
-        if w is None:
-            w = weave.Weave(file_id)
-            self.text_weaves[file_id] = w
-        text_changed = False
-        parent_candiate_entries = ie.parent_candidates(parent_invs)
-        heads = graph.Graph(self).heads(parent_candiate_entries.keys())
-        # XXX: Note that this is unordered - and this is tolerable because
-        # the previous code was also unordered.
-        previous_entries = dict((head, parent_candiate_entries[head]) for head
-            in heads)
-        self.snapshot_ie(previous_entries, ie, w, rev_id)
-
-    def get_parent_map(self, revision_ids):
-        """See graph.StackedParentsProvider.get_parent_map"""
-        return dict((revision_id, self.revisions[revision_id])
-                    for revision_id in revision_ids
-                     if revision_id in self.revisions)
-
-    def snapshot_ie(self, previous_revisions, ie, w, rev_id):
-        # TODO: convert this logic, which is ~= snapshot to
-        # a call to:. This needs the path figured out. rather than a work_tree
-        # a v4 revision_tree can be given, or something that looks enough like
-        # one to give the file content to the entry if it needs it.
-        # and we need something that looks like a weave store for snapshot to
-        # save against.
-        #ie.snapshot(rev, PATH, previous_revisions, REVISION_TREE, InMemoryWeaveStore(self.text_weaves))
-        if len(previous_revisions) == 1:
-            previous_ie = previous_revisions.values()[0]
-            if ie._unchanged(previous_ie):
-                ie.revision = previous_ie.revision
-                return
-        if ie.has_text():
-            f = self.branch.repository._text_store.get(ie.text_id)
-            try:
-                file_lines = f.readlines()
-            finally:
-                f.close()
-            w.add_lines(rev_id, previous_revisions, file_lines)
-            self.text_count += 1
-        else:
-            w.add_lines(rev_id, previous_revisions, [])
-        ie.revision = rev_id
-
-    def _make_order(self):
-        """Return a suitable order for importing revisions.
-
-        The order must be such that an revision is imported after all
-        its (present) parents.
-        """
-        todo = set(self.revisions.keys())
-        done = self.absent_revisions.copy()
-        order = []
-        while todo:
-            # scan through looking for a revision whose parents
-            # are all done
-            for rev_id in sorted(list(todo)):
-                rev = self.revisions[rev_id]
-                parent_ids = set(rev.parent_ids)
-                if parent_ids.issubset(done):
-                    # can take this one now
-                    order.append(rev_id)
-                    todo.remove(rev_id)
-                    done.add(rev_id)
-        return order
-
-
-
-
-class ConvertBzrDir5To6(Converter):
-    """Converts format 5 bzr dirs to format 6."""
-
-    def convert(self, to_convert, pb):
-        """See Converter.convert()."""
-        self.bzrdir = to_convert
-        pb = ui.ui_factory.nested_progress_bar()
-        try:
-            ui.ui_factory.note('starting upgrade from format 5 to 6')
-            self._convert_to_prefixed()
-            return BzrDir.open(self.bzrdir.user_url)
-        finally:
-            pb.finished()
-
-    def _convert_to_prefixed(self):
-        from bzrlib.store import TransportStore
-        self.bzrdir.transport.delete('branch-format')
-        for store_name in ["weaves", "revision-store"]:
-            ui.ui_factory.note("adding prefixes to %s" % store_name)
-            store_transport = self.bzrdir.transport.clone(store_name)
-            store = TransportStore(store_transport, prefixed=True)
-            for urlfilename in store_transport.list_dir('.'):
-                filename = urlutils.unescape(urlfilename)
-                if (filename.endswith(".weave") or
-                    filename.endswith(".gz") or
-                    filename.endswith(".sig")):
-                    file_id, suffix = os.path.splitext(filename)
-                else:
-                    file_id = filename
-                    suffix = ''
-                new_name = store._mapper.map((file_id,)) + suffix
-                # FIXME keep track of the dirs made RBC 20060121
-                try:
-                    store_transport.move(filename, new_name)
-                except errors.NoSuchFile: # catches missing dirs strangely enough
-                    store_transport.mkdir(osutils.dirname(new_name))
-                    store_transport.move(filename, new_name)
-        self.bzrdir.transport.put_bytes(
-            'branch-format',
-            BzrDirFormat6().get_format_string(),
-            mode=self.bzrdir._get_file_mode())
-
-
-class ConvertBzrDir6ToMeta(Converter):
-    """Converts format 6 bzr dirs to metadirs."""
-
-    def convert(self, to_convert, pb):
-        """See Converter.convert()."""
-        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat7
-        from bzrlib.branch import BzrBranchFormat5
-        self.bzrdir = to_convert
-        self.pb = ui.ui_factory.nested_progress_bar()
-        self.count = 0
-        self.total = 20 # the steps we know about
-        self.garbage_inventories = []
-        self.dir_mode = self.bzrdir._get_dir_mode()
-        self.file_mode = self.bzrdir._get_file_mode()
-
-        ui.ui_factory.note('starting upgrade from format 6 to metadir')
-        self.bzrdir.transport.put_bytes(
-                'branch-format',
-                "Converting to format 6",
-                mode=self.file_mode)
-        # its faster to move specific files around than to open and use the apis...
-        # first off, nuke ancestry.weave, it was never used.
-        try:
-            self.step('Removing ancestry.weave')
-            self.bzrdir.transport.delete('ancestry.weave')
-        except errors.NoSuchFile:
-            pass
-        # find out whats there
-        self.step('Finding branch files')
-        last_revision = self.bzrdir.open_branch().last_revision()
-        bzrcontents = self.bzrdir.transport.list_dir('.')
-        for name in bzrcontents:
-            if name.startswith('basis-inventory.'):
-                self.garbage_inventories.append(name)
-        # create new directories for repository, working tree and branch
-        repository_names = [('inventory.weave', True),
-                            ('revision-store', True),
-                            ('weaves', True)]
-        self.step('Upgrading repository  ')
-        self.bzrdir.transport.mkdir('repository', mode=self.dir_mode)
-        self.make_lock('repository')
-        # we hard code the formats here because we are converting into
-        # the meta format. The meta format upgrader can take this to a
-        # future format within each component.
-        self.put_format('repository', RepositoryFormat7())
-        for entry in repository_names:
-            self.move_entry('repository', entry)
-
-        self.step('Upgrading branch      ')
-        self.bzrdir.transport.mkdir('branch', mode=self.dir_mode)
-        self.make_lock('branch')
-        self.put_format('branch', BzrBranchFormat5())
-        branch_files = [('revision-history', True),
-                        ('branch-name', True),
-                        ('parent', False)]
-        for entry in branch_files:
-            self.move_entry('branch', entry)
-
-        checkout_files = [('pending-merges', True),
-                          ('inventory', True),
-                          ('stat-cache', False)]
-        # If a mandatory checkout file is not present, the branch does not have
-        # a functional checkout. Do not create a checkout in the converted
-        # branch.
-        for name, mandatory in checkout_files:
-            if mandatory and name not in bzrcontents:
-                has_checkout = False
-                break
-        else:
-            has_checkout = True
-        if not has_checkout:
-            ui.ui_factory.note('No working tree.')
-            # If some checkout files are there, we may as well get rid of them.
-            for name, mandatory in checkout_files:
-                if name in bzrcontents:
-                    self.bzrdir.transport.delete(name)
-        else:
-            from bzrlib.workingtree import WorkingTreeFormat3
-            self.step('Upgrading working tree')
-            self.bzrdir.transport.mkdir('checkout', mode=self.dir_mode)
-            self.make_lock('checkout')
-            self.put_format(
-                'checkout', WorkingTreeFormat3())
-            self.bzrdir.transport.delete_multi(
-                self.garbage_inventories, self.pb)
-            for entry in checkout_files:
-                self.move_entry('checkout', entry)
-            if last_revision is not None:
-                self.bzrdir.transport.put_bytes(
-                    'checkout/last-revision', last_revision)
-        self.bzrdir.transport.put_bytes(
-            'branch-format',
-            BzrDirMetaFormat1().get_format_string(),
-            mode=self.file_mode)
-        self.pb.finished()
-        return BzrDir.open(self.bzrdir.user_url)
-
-    def make_lock(self, name):
-        """Make a lock for the new control dir name."""
-        self.step('Make %s lock' % name)
-        ld = lockdir.LockDir(self.bzrdir.transport,
-                             '%s/lock' % name,
-                             file_modebits=self.file_mode,
-                             dir_modebits=self.dir_mode)
-        ld.create()
-
-    def move_entry(self, new_dir, entry):
-        """Move then entry name into new_dir."""
-        name = entry[0]
-        mandatory = entry[1]
-        self.step('Moving %s' % name)
-        try:
-            self.bzrdir.transport.move(name, '%s/%s' % (new_dir, name))
-        except errors.NoSuchFile:
-            if mandatory:
-                raise
-
-    def put_format(self, dirname, format):
-        self.bzrdir.transport.put_bytes('%s/format' % dirname,
-            format.get_format_string(),
-            self.file_mode)
-
-
-class BzrDirFormat4(BzrDirFormat):
-    """Bzr dir format 4.
-
-    This format is a combined format for working tree, branch and repository.
-    It has:
-     - Format 1 working trees [always]
-     - Format 4 branches [always]
-     - Format 4 repositories [always]
-
-    This format is deprecated: it indexes texts using a text it which is
-    removed in format 5; write support for this format has been removed.
-    """
-
-    _lock_class = lockable_files.TransportLock
-
-    def get_format_string(self):
-        """See BzrDirFormat.get_format_string()."""
-        return "Bazaar-NG branch, format 0.0.4\n"
-
-    def get_format_description(self):
-        """See BzrDirFormat.get_format_description()."""
-        return "All-in-one format 4"
-
-    def get_converter(self, format=None):
-        """See BzrDirFormat.get_converter()."""
-        # there is one and only one upgrade path here.
-        return ConvertBzrDir4To5()
-
-    def initialize_on_transport(self, transport):
-        """Format 4 branches cannot be created."""
-        raise errors.UninitializableFormat(self)
-
-    def is_supported(self):
-        """Format 4 is not supported.
-
-        It is not supported because the model changed from 4 to 5 and the
-        conversion logic is expensive - so doing it on the fly was not
-        feasible.
-        """
-        return False
-
-    def network_name(self):
-        return self.get_format_string()
-
-    def _open(self, transport):
-        """See BzrDirFormat._open."""
-        return BzrDir4(transport, self)
-
-    def __return_repository_format(self):
-        """Circular import protection."""
-        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat4
-        return RepositoryFormat4()
-    repository_format = property(__return_repository_format)
-
-
-class BzrDirPreSplitOut(BzrDir):
-    """A common class for the all-in-one formats."""
-
-    def __init__(self, _transport, _format):
-        """See BzrDir.__init__."""
-        super(BzrDirPreSplitOut, self).__init__(_transport, _format)
-        self._control_files = lockable_files.LockableFiles(
-                                            self.get_branch_transport(None),
-                                            self._format._lock_file_name,
-                                            self._format._lock_class)
-
-    def break_lock(self):
-        """Pre-splitout bzrdirs do not suffer from stale locks."""
-        raise NotImplementedError(self.break_lock)
-
-    def cloning_metadir(self, require_stacking=False):
-        """Produce a metadir suitable for cloning with."""
-        if require_stacking:
-            return format_registry.make_bzrdir('1.6')
-        return self._format.__class__()
-
-    def clone(self, url, revision_id=None, force_new_repo=False,
-              preserve_stacking=False):
-        """See BzrDir.clone().
-
-        force_new_repo has no effect, since this family of formats always
-        require a new repository.
-        preserve_stacking has no effect, since no source branch using this
-        family of formats can be stacked, so there is no stacking to preserve.
-        """
-        self._make_tail(url)
-        result = self._format._initialize_for_clone(url)
-        self.open_repository().clone(result, revision_id=revision_id)
-        from_branch = self.open_branch()
-        from_branch.clone(result, revision_id=revision_id)
-        try:
-            tree = self.open_workingtree()
-        except errors.NotLocalUrl:
-            # make a new one, this format always has to have one.
-            result._init_workingtree()
-        else:
-            tree.clone(result)
-        return result
-
-    def create_branch(self, name=None, repository=None):
-        """See BzrDir.create_branch."""
-        if repository is not None:
-            raise NotImplementedError(
-                "create_branch(repository=<not None>) on %r" % (self,))
-        return self._format.get_branch_format().initialize(self, name=name)
-
-    def destroy_branch(self, name=None):
-        """See BzrDir.destroy_branch."""
-        raise errors.UnsupportedOperation(self.destroy_branch, self)
-
-    def create_repository(self, shared=False):
-        """See BzrDir.create_repository."""
-        if shared:
-            raise errors.IncompatibleFormat('shared repository', self._format)
-        return self.open_repository()
-
-    def destroy_repository(self):
-        """See BzrDir.destroy_repository."""
-        raise errors.UnsupportedOperation(self.destroy_repository, self)
-
-    def create_workingtree(self, revision_id=None, from_branch=None,
-                           accelerator_tree=None, hardlink=False):
-        """See BzrDir.create_workingtree."""
-        # The workingtree is sometimes created when the bzrdir is created,
-        # but not when cloning.
-
-        # this looks buggy but is not -really-
-        # because this format creates the workingtree when the bzrdir is
-        # created
-        # clone and sprout will have set the revision_id
-        # and that will have set it for us, its only
-        # specific uses of create_workingtree in isolation
-        # that can do wonky stuff here, and that only
-        # happens for creating checkouts, which cannot be
-        # done on this format anyway. So - acceptable wart.
-        if hardlink:
-            warning("can't support hardlinked working trees in %r"
-                % (self,))
-        try:
-            result = self.open_workingtree(recommend_upgrade=False)
-        except errors.NoSuchFile:
-            result = self._init_workingtree()
-        if revision_id is not None:
-            if revision_id == _mod_revision.NULL_REVISION:
-                result.set_parent_ids([])
-            else:
-                result.set_parent_ids([revision_id])
-        return result
-
-    def _init_workingtree(self):
-        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
-        try:
-            return WorkingTreeFormat2().initialize(self)
-        except errors.NotLocalUrl:
-            # Even though we can't access the working tree, we need to
-            # create its control files.
-            return WorkingTreeFormat2()._stub_initialize_on_transport(
-                self.transport, self._control_files._file_mode)
-
-    def destroy_workingtree(self):
-        """See BzrDir.destroy_workingtree."""
-        raise errors.UnsupportedOperation(self.destroy_workingtree, self)
-
-    def destroy_workingtree_metadata(self):
-        """See BzrDir.destroy_workingtree_metadata."""
-        raise errors.UnsupportedOperation(self.destroy_workingtree_metadata,
-                                          self)
-
-    def get_branch_transport(self, branch_format, name=None):
-        """See BzrDir.get_branch_transport()."""
-        if name is not None:
-            raise errors.NoColocatedBranchSupport(self)
-        if branch_format is None:
-            return self.transport
-        try:
-            branch_format.get_format_string()
-        except NotImplementedError:
-            return self.transport
-        raise errors.IncompatibleFormat(branch_format, self._format)
-
-    def get_repository_transport(self, repository_format):
-        """See BzrDir.get_repository_transport()."""
-        if repository_format is None:
-            return self.transport
-        try:
-            repository_format.get_format_string()
-        except NotImplementedError:
-            return self.transport
-        raise errors.IncompatibleFormat(repository_format, self._format)
-
-    def get_workingtree_transport(self, workingtree_format):
-        """See BzrDir.get_workingtree_transport()."""
-        if workingtree_format is None:
-            return self.transport
-        try:
-            workingtree_format.get_format_string()
-        except NotImplementedError:
-            return self.transport
-        raise errors.IncompatibleFormat(workingtree_format, self._format)
-
-    def needs_format_conversion(self, format=None):
-        """See BzrDir.needs_format_conversion()."""
-        # if the format is not the same as the system default,
-        # an upgrade is needed.
-        if format is None:
-            symbol_versioning.warn(symbol_versioning.deprecated_in((1, 13, 0))
-                % 'needs_format_conversion(format=None)')
-            format = BzrDirFormat.get_default_format()
-        return not isinstance(self._format, format.__class__)
-
-    def open_branch(self, name=None, unsupported=False,
-                    ignore_fallbacks=False):
-        """See BzrDir.open_branch."""
-        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
-        format = BzrBranchFormat4()
-        self._check_supported(format, unsupported)
-        return format.open(self, name, _found=True)
-
-    def sprout(self, url, revision_id=None, force_new_repo=False,
-               possible_transports=None, accelerator_tree=None,
-               hardlink=False, stacked=False, create_tree_if_local=True,
-               source_branch=None):
-        """See BzrDir.sprout()."""
-        if source_branch is not None:
-            my_branch = self.open_branch()
-            if source_branch.base != my_branch.base:
-                raise AssertionError(
-                    "source branch %r is not within %r with branch %r" %
-                    (source_branch, self, my_branch))
-        if stacked:
-            raise errors.UnstackableBranchFormat(
-                self._format, self.root_transport.base)
-        if not create_tree_if_local:
-            raise errors.MustHaveWorkingTree(
-                self._format, self.root_transport.base)
-        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
-        self._make_tail(url)
-        result = self._format._initialize_for_clone(url)
-        try:
-            self.open_repository().clone(result, revision_id=revision_id)
-        except errors.NoRepositoryPresent:
-            pass
-        try:
-            self.open_branch().sprout(result, revision_id=revision_id)
-        except errors.NotBranchError:
-            pass
-
-        # we always want a working tree
-        WorkingTreeFormat2().initialize(result,
-                                        accelerator_tree=accelerator_tree,
-                                        hardlink=hardlink)
-        return result
-
-
-class BzrDir4(BzrDirPreSplitOut):
-    """A .bzr version 4 control object.
-
-    This is a deprecated format and may be removed after sept 2006.
-    """
-
-    def create_repository(self, shared=False):
-        """See BzrDir.create_repository."""
-        return self._format.repository_format.initialize(self, shared)
-
-    def needs_format_conversion(self, format=None):
-        """Format 4 dirs are always in need of conversion."""
-        if format is None:
-            symbol_versioning.warn(symbol_versioning.deprecated_in((1, 13, 0))
-                % 'needs_format_conversion(format=None)')
-        return True
-
-    def open_repository(self):
-        """See BzrDir.open_repository."""
-        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat4
-        return RepositoryFormat4().open(self, _found=True)
-
-
-
-
-class BzrDir5(BzrDirPreSplitOut):
-    """A .bzr version 5 control object.
-
-    This is a deprecated format and may be removed after sept 2006.
-    """
-
-    def has_workingtree(self):
-        """See BzrDir.has_workingtree."""
-        return True
-    
-    def open_repository(self):
-        """See BzrDir.open_repository."""
-        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat5
-        return RepositoryFormat5().open(self, _found=True)
-
-    def open_workingtree(self, _unsupported=False,
-            recommend_upgrade=True):
-        """See BzrDir.create_workingtree."""
-        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
-        wt_format = WorkingTreeFormat2()
-        # we don't warn here about upgrades; that ought to be handled for the
-        # bzrdir as a whole
-        return wt_format.open(self, _found=True)
-
-
-class BzrDir6(BzrDirPreSplitOut):
-    """A .bzr version 6 control object.
-
-    This is a deprecated format and may be removed after sept 2006.
-    """
-
-    def has_workingtree(self):
-        """See BzrDir.has_workingtree."""
-        return True
-
-    def open_repository(self):
-        """See BzrDir.open_repository."""
-        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat6
-        return RepositoryFormat6().open(self, _found=True)
-
-    def open_workingtree(self, _unsupported=False,
-        recommend_upgrade=True):
-        """See BzrDir.create_workingtree."""
-        # we don't warn here about upgrades; that ought to be handled for the
-        # bzrdir as a whole
-        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
-        return WorkingTreeFormat2().open(self, _found=True)

=== added file 'bzrlib/plugins/weave_fmt/bzrdir.py'
--- bzrlib/plugins/weave_fmt/bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/weave_fmt/bzrdir.py	2011-03-10 13:29:54 +0000
@@ -0,0 +1,969 @@
+# Copyright (C) 2006-2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Weave-era BzrDir formats."""
+
+from bzrlib.bzrdir import (
+    BzrDir,
+    BzrDirFormat,
+    BzrDirMetaFormat1,
+    )
+from bzrlib.controldir import (
+    Converter,
+    format_registry,
+    )
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import os
+import warnings
+
+from bzrlib import (
+    errors,
+    graph,
+    lockable_files,
+    lockdir,
+    osutils,
+    revision as _mod_revision,
+    trace,
+    ui,
+    urlutils,
+    versionedfile,
+    weave,
+    xml5,
+    )
+from bzrlib.store.versioned import VersionedFileStore
+from bzrlib.transactions import WriteTransaction
+from bzrlib.transport import (
+    get_transport,
+    local,
+    )
+from bzrlib.plugins.weave_fmt import xml4
+""")
+
+
+class BzrDirFormatAllInOne(BzrDirFormat):
+    """Common class for formats before meta-dirs."""
+
+    fixed_components = True
+
+    def initialize_on_transport_ex(self, transport, use_existing_dir=False,
+        create_prefix=False, force_new_repo=False, stacked_on=None,
+        stack_on_pwd=None, repo_format_name=None, make_working_trees=None,
+        shared_repo=False):
+        """See BzrDirFormat.initialize_on_transport_ex."""
+        require_stacking = (stacked_on is not None)
+        # Format 5 cannot stack, but we've been asked to - actually init
+        # a Meta1Dir
+        if require_stacking:
+            format = BzrDirMetaFormat1()
+            return format.initialize_on_transport_ex(transport,
+                use_existing_dir=use_existing_dir, create_prefix=create_prefix,
+                force_new_repo=force_new_repo, stacked_on=stacked_on,
+                stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
+                make_working_trees=make_working_trees, shared_repo=shared_repo)
+        return BzrDirFormat.initialize_on_transport_ex(self, transport,
+            use_existing_dir=use_existing_dir, create_prefix=create_prefix,
+            force_new_repo=force_new_repo, stacked_on=stacked_on,
+            stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
+            make_working_trees=make_working_trees, shared_repo=shared_repo)
+
+
+class BzrDirFormat5(BzrDirFormatAllInOne):
+    """Bzr control format 5.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - Format 2 working trees [always]
+     - Format 4 branches [always]
+     - Format 5 repositories [always]
+       Unhashed stores in the repository.
+    """
+
+    _lock_class = lockable_files.TransportLock
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 5\n"
+
+    def get_branch_format(self):
+        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
+        return BzrBranchFormat4()
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "All-in-one format 5"
+
+    def get_converter(self, format=None):
+        """See BzrDirFormat.get_converter()."""
+        # there is one and only one upgrade path here.
+        return ConvertBzrDir5To6()
+
+    def _initialize_for_clone(self, url):
+        return self.initialize_on_transport(get_transport(url), _cloning=True)
+
+    def initialize_on_transport(self, transport, _cloning=False):
+        """Format 5 dirs always have working tree, branch and repository.
+
+        Except when they are being cloned.
+        """
+        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat5
+        result = (super(BzrDirFormat5, self).initialize_on_transport(transport))
+        RepositoryFormat5().initialize(result, _internal=True)
+        if not _cloning:
+            branch = BzrBranchFormat4().initialize(result)
+            result._init_workingtree()
+        return result
+
+    def network_name(self):
+        return self.get_format_string()
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir5(transport, self)
+
+    def __return_repository_format(self):
+        """Circular import protection."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat5
+        return RepositoryFormat5()
+    repository_format = property(__return_repository_format)
+
+
+class BzrDirFormat6(BzrDirFormatAllInOne):
+    """Bzr control format 6.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - Format 2 working trees [always]
+     - Format 4 branches [always]
+     - Format 6 repositories [always]
+    """
+
+    _lock_class = lockable_files.TransportLock
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 6\n"
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "All-in-one format 6"
+
+    def get_branch_format(self):
+        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
+        return BzrBranchFormat4()
+
+    def get_converter(self, format=None):
+        """See BzrDirFormat.get_converter()."""
+        # there is one and only one upgrade path here.
+        return ConvertBzrDir6ToMeta()
+
+    def _initialize_for_clone(self, url):
+        return self.initialize_on_transport(get_transport(url), _cloning=True)
+
+    def initialize_on_transport(self, transport, _cloning=False):
+        """Format 6 dirs always have working tree, branch and repository.
+
+        Except when they are being cloned.
+        """
+        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat6
+        result = super(BzrDirFormat6, self).initialize_on_transport(transport)
+        RepositoryFormat6().initialize(result, _internal=True)
+        if not _cloning:
+            branch = BzrBranchFormat4().initialize(result)
+            result._init_workingtree()
+        return result
+
+    def network_name(self):
+        return self.get_format_string()
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir6(transport, self)
+
+    def __return_repository_format(self):
+        """Circular import protection."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat6
+        return RepositoryFormat6()
+    repository_format = property(__return_repository_format)
+
+
+class ConvertBzrDir4To5(Converter):
+    """Converts format 4 bzr dirs to format 5."""
+
+    def __init__(self):
+        super(ConvertBzrDir4To5, self).__init__()
+        self.converted_revs = set()
+        self.absent_revisions = set()
+        self.text_count = 0
+        self.revisions = {}
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        self.bzrdir = to_convert
+        if pb is not None:
+            warnings.warn("pb parameter to convert() is deprecated")
+        self.pb = ui.ui_factory.nested_progress_bar()
+        try:
+            ui.ui_factory.note('starting upgrade from format 4 to 5')
+            if isinstance(self.bzrdir.transport, local.LocalTransport):
+                self.bzrdir.get_workingtree_transport(None).delete('stat-cache')
+            self._convert_to_weaves()
+            return BzrDir.open(self.bzrdir.user_url)
+        finally:
+            self.pb.finished()
+
+    def _convert_to_weaves(self):
+        ui.ui_factory.note('note: upgrade may be faster if all store files are ungzipped first')
+        try:
+            # TODO permissions
+            stat = self.bzrdir.transport.stat('weaves')
+            if not S_ISDIR(stat.st_mode):
+                self.bzrdir.transport.delete('weaves')
+                self.bzrdir.transport.mkdir('weaves')
+        except errors.NoSuchFile:
+            self.bzrdir.transport.mkdir('weaves')
+        # deliberately not a WeaveFile as we want to build it up slowly.
+        self.inv_weave = weave.Weave('inventory')
+        # holds in-memory weaves for all files
+        self.text_weaves = {}
+        self.bzrdir.transport.delete('branch-format')
+        self.branch = self.bzrdir.open_branch()
+        self._convert_working_inv()
+        rev_history = self.branch.revision_history()
+        # to_read is a stack holding the revisions we still need to process;
+        # appending to it adds new highest-priority revisions
+        self.known_revisions = set(rev_history)
+        self.to_read = rev_history[-1:]
+        while self.to_read:
+            rev_id = self.to_read.pop()
+            if (rev_id not in self.revisions
+                and rev_id not in self.absent_revisions):
+                self._load_one_rev(rev_id)
+        self.pb.clear()
+        to_import = self._make_order()
+        for i, rev_id in enumerate(to_import):
+            self.pb.update('converting revision', i, len(to_import))
+            self._convert_one_rev(rev_id)
+        self.pb.clear()
+        self._write_all_weaves()
+        self._write_all_revs()
+        ui.ui_factory.note('upgraded to weaves:')
+        ui.ui_factory.note('  %6d revisions and inventories' % len(self.revisions))
+        ui.ui_factory.note('  %6d revisions not present' % len(self.absent_revisions))
+        ui.ui_factory.note('  %6d texts' % self.text_count)
+        self._cleanup_spare_files_after_format4()
+        self.branch._transport.put_bytes(
+            'branch-format',
+            BzrDirFormat5().get_format_string(),
+            mode=self.bzrdir._get_file_mode())
+
+    def _cleanup_spare_files_after_format4(self):
+        # FIXME working tree upgrade foo.
+        for n in 'merged-patches', 'pending-merged-patches':
+            try:
+                ## assert os.path.getsize(p) == 0
+                self.bzrdir.transport.delete(n)
+            except errors.NoSuchFile:
+                pass
+        self.bzrdir.transport.delete_tree('inventory-store')
+        self.bzrdir.transport.delete_tree('text-store')
+
+    def _convert_working_inv(self):
+        inv = xml4.serializer_v4.read_inventory(
+                self.branch._transport.get('inventory'))
+        new_inv_xml = xml5.serializer_v5.write_inventory_to_string(inv, working=True)
+        self.branch._transport.put_bytes('inventory', new_inv_xml,
+            mode=self.bzrdir._get_file_mode())
+
+    def _write_all_weaves(self):
+        controlweaves = VersionedFileStore(self.bzrdir.transport, prefixed=False,
+            versionedfile_class=weave.WeaveFile)
+        weave_transport = self.bzrdir.transport.clone('weaves')
+        weaves = VersionedFileStore(weave_transport, prefixed=False,
+                versionedfile_class=weave.WeaveFile)
+        transaction = WriteTransaction()
+
+        try:
+            i = 0
+            for file_id, file_weave in self.text_weaves.items():
+                self.pb.update('writing weave', i, len(self.text_weaves))
+                weaves._put_weave(file_id, file_weave, transaction)
+                i += 1
+            self.pb.update('inventory', 0, 1)
+            controlweaves._put_weave('inventory', self.inv_weave, transaction)
+            self.pb.update('inventory', 1, 1)
+        finally:
+            self.pb.clear()
+
+    def _write_all_revs(self):
+        """Write all revisions out in new form."""
+        self.bzrdir.transport.delete_tree('revision-store')
+        self.bzrdir.transport.mkdir('revision-store')
+        revision_transport = self.bzrdir.transport.clone('revision-store')
+        # TODO permissions
+        from bzrlib.xml5 import serializer_v5
+        from bzrlib.plugins.weave_fmt.repository import RevisionTextStore
+        revision_store = RevisionTextStore(revision_transport,
+            serializer_v5, False, versionedfile.PrefixMapper(),
+            lambda:True, lambda:True)
+        try:
+            for i, rev_id in enumerate(self.converted_revs):
+                self.pb.update('write revision', i, len(self.converted_revs))
+                text = serializer_v5.write_revision_to_string(
+                    self.revisions[rev_id])
+                key = (rev_id,)
+                revision_store.add_lines(key, None, osutils.split_lines(text))
+        finally:
+            self.pb.clear()
+
+    def _load_one_rev(self, rev_id):
+        """Load a revision object into memory.
+
+        Any parents not either loaded or abandoned get queued to be
+        loaded."""
+        self.pb.update('loading revision',
+                       len(self.revisions),
+                       len(self.known_revisions))
+        if not self.branch.repository.has_revision(rev_id):
+            self.pb.clear()
+            ui.ui_factory.note('revision {%s} not present in branch; '
+                         'will be converted as a ghost' %
+                         rev_id)
+            self.absent_revisions.add(rev_id)
+        else:
+            rev = self.branch.repository.get_revision(rev_id)
+            for parent_id in rev.parent_ids:
+                self.known_revisions.add(parent_id)
+                self.to_read.append(parent_id)
+            self.revisions[rev_id] = rev
+
+    def _load_old_inventory(self, rev_id):
+        f = self.branch.repository.inventory_store.get(rev_id)
+        try:
+            old_inv_xml = f.read()
+        finally:
+            f.close()
+        inv = xml4.serializer_v4.read_inventory_from_string(old_inv_xml)
+        inv.revision_id = rev_id
+        rev = self.revisions[rev_id]
+        return inv
+
+    def _load_updated_inventory(self, rev_id):
+        inv_xml = self.inv_weave.get_text(rev_id)
+        inv = xml5.serializer_v5.read_inventory_from_string(inv_xml, rev_id)
+        return inv
+
+    def _convert_one_rev(self, rev_id):
+        """Convert revision and all referenced objects to new format."""
+        rev = self.revisions[rev_id]
+        inv = self._load_old_inventory(rev_id)
+        present_parents = [p for p in rev.parent_ids
+                           if p not in self.absent_revisions]
+        self._convert_revision_contents(rev, inv, present_parents)
+        self._store_new_inv(rev, inv, present_parents)
+        self.converted_revs.add(rev_id)
+
+    def _store_new_inv(self, rev, inv, present_parents):
+        new_inv_xml = xml5.serializer_v5.write_inventory_to_string(inv)
+        new_inv_sha1 = osutils.sha_string(new_inv_xml)
+        self.inv_weave.add_lines(rev.revision_id,
+                                 present_parents,
+                                 new_inv_xml.splitlines(True))
+        rev.inventory_sha1 = new_inv_sha1
+
+    def _convert_revision_contents(self, rev, inv, present_parents):
+        """Convert all the files within a revision.
+
+        Also upgrade the inventory to refer to the text revision ids."""
+        rev_id = rev.revision_id
+        trace.mutter('converting texts of revision {%s}', rev_id)
+        parent_invs = map(self._load_updated_inventory, present_parents)
+        entries = inv.iter_entries()
+        entries.next()
+        for path, ie in entries:
+            self._convert_file_version(rev, ie, parent_invs)
+
+    def _convert_file_version(self, rev, ie, parent_invs):
+        """Convert one version of one file.
+
+        The file needs to be added into the weave if it is a merge
+        of >=2 parents or if it's changed from its parent.
+        """
+        file_id = ie.file_id
+        rev_id = rev.revision_id
+        w = self.text_weaves.get(file_id)
+        if w is None:
+            w = weave.Weave(file_id)
+            self.text_weaves[file_id] = w
+        text_changed = False
+        parent_candiate_entries = ie.parent_candidates(parent_invs)
+        heads = graph.Graph(self).heads(parent_candiate_entries.keys())
+        # XXX: Note that this is unordered - and this is tolerable because
+        # the previous code was also unordered.
+        previous_entries = dict((head, parent_candiate_entries[head]) for head
+            in heads)
+        self.snapshot_ie(previous_entries, ie, w, rev_id)
+
+    def get_parent_map(self, revision_ids):
+        """See graph.StackedParentsProvider.get_parent_map"""
+        return dict((revision_id, self.revisions[revision_id])
+                    for revision_id in revision_ids
+                     if revision_id in self.revisions)
+
+    def snapshot_ie(self, previous_revisions, ie, w, rev_id):
+        # TODO: convert this logic, which is ~= snapshot to
+        # a call to:. This needs the path figured out. rather than a work_tree
+        # a v4 revision_tree can be given, or something that looks enough like
+        # one to give the file content to the entry if it needs it.
+        # and we need something that looks like a weave store for snapshot to
+        # save against.
+        #ie.snapshot(rev, PATH, previous_revisions, REVISION_TREE, InMemoryWeaveStore(self.text_weaves))
+        if len(previous_revisions) == 1:
+            previous_ie = previous_revisions.values()[0]
+            if ie._unchanged(previous_ie):
+                ie.revision = previous_ie.revision
+                return
+        if ie.has_text():
+            f = self.branch.repository._text_store.get(ie.text_id)
+            try:
+                file_lines = f.readlines()
+            finally:
+                f.close()
+            w.add_lines(rev_id, previous_revisions, file_lines)
+            self.text_count += 1
+        else:
+            w.add_lines(rev_id, previous_revisions, [])
+        ie.revision = rev_id
+
+    def _make_order(self):
+        """Return a suitable order for importing revisions.
+
+        The order must be such that an revision is imported after all
+        its (present) parents.
+        """
+        todo = set(self.revisions.keys())
+        done = self.absent_revisions.copy()
+        order = []
+        while todo:
+            # scan through looking for a revision whose parents
+            # are all done
+            for rev_id in sorted(list(todo)):
+                rev = self.revisions[rev_id]
+                parent_ids = set(rev.parent_ids)
+                if parent_ids.issubset(done):
+                    # can take this one now
+                    order.append(rev_id)
+                    todo.remove(rev_id)
+                    done.add(rev_id)
+        return order
+
+
+
+
+class ConvertBzrDir5To6(Converter):
+    """Converts format 5 bzr dirs to format 6."""
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        self.bzrdir = to_convert
+        pb = ui.ui_factory.nested_progress_bar()
+        try:
+            ui.ui_factory.note('starting upgrade from format 5 to 6')
+            self._convert_to_prefixed()
+            return BzrDir.open(self.bzrdir.user_url)
+        finally:
+            pb.finished()
+
+    def _convert_to_prefixed(self):
+        from bzrlib.store import TransportStore
+        self.bzrdir.transport.delete('branch-format')
+        for store_name in ["weaves", "revision-store"]:
+            ui.ui_factory.note("adding prefixes to %s" % store_name)
+            store_transport = self.bzrdir.transport.clone(store_name)
+            store = TransportStore(store_transport, prefixed=True)
+            for urlfilename in store_transport.list_dir('.'):
+                filename = urlutils.unescape(urlfilename)
+                if (filename.endswith(".weave") or
+                    filename.endswith(".gz") or
+                    filename.endswith(".sig")):
+                    file_id, suffix = os.path.splitext(filename)
+                else:
+                    file_id = filename
+                    suffix = ''
+                new_name = store._mapper.map((file_id,)) + suffix
+                # FIXME keep track of the dirs made RBC 20060121
+                try:
+                    store_transport.move(filename, new_name)
+                except errors.NoSuchFile: # catches missing dirs strangely enough
+                    store_transport.mkdir(osutils.dirname(new_name))
+                    store_transport.move(filename, new_name)
+        self.bzrdir.transport.put_bytes(
+            'branch-format',
+            BzrDirFormat6().get_format_string(),
+            mode=self.bzrdir._get_file_mode())
+
+
+class ConvertBzrDir6ToMeta(Converter):
+    """Converts format 6 bzr dirs to metadirs."""
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat7
+        from bzrlib.branch import BzrBranchFormat5
+        self.bzrdir = to_convert
+        self.pb = ui.ui_factory.nested_progress_bar()
+        self.count = 0
+        self.total = 20 # the steps we know about
+        self.garbage_inventories = []
+        self.dir_mode = self.bzrdir._get_dir_mode()
+        self.file_mode = self.bzrdir._get_file_mode()
+
+        ui.ui_factory.note('starting upgrade from format 6 to metadir')
+        self.bzrdir.transport.put_bytes(
+                'branch-format',
+                "Converting to format 6",
+                mode=self.file_mode)
+        # its faster to move specific files around than to open and use the apis...
+        # first off, nuke ancestry.weave, it was never used.
+        try:
+            self.step('Removing ancestry.weave')
+            self.bzrdir.transport.delete('ancestry.weave')
+        except errors.NoSuchFile:
+            pass
+        # find out whats there
+        self.step('Finding branch files')
+        last_revision = self.bzrdir.open_branch().last_revision()
+        bzrcontents = self.bzrdir.transport.list_dir('.')
+        for name in bzrcontents:
+            if name.startswith('basis-inventory.'):
+                self.garbage_inventories.append(name)
+        # create new directories for repository, working tree and branch
+        repository_names = [('inventory.weave', True),
+                            ('revision-store', True),
+                            ('weaves', True)]
+        self.step('Upgrading repository  ')
+        self.bzrdir.transport.mkdir('repository', mode=self.dir_mode)
+        self.make_lock('repository')
+        # we hard code the formats here because we are converting into
+        # the meta format. The meta format upgrader can take this to a
+        # future format within each component.
+        self.put_format('repository', RepositoryFormat7())
+        for entry in repository_names:
+            self.move_entry('repository', entry)
+
+        self.step('Upgrading branch      ')
+        self.bzrdir.transport.mkdir('branch', mode=self.dir_mode)
+        self.make_lock('branch')
+        self.put_format('branch', BzrBranchFormat5())
+        branch_files = [('revision-history', True),
+                        ('branch-name', True),
+                        ('parent', False)]
+        for entry in branch_files:
+            self.move_entry('branch', entry)
+
+        checkout_files = [('pending-merges', True),
+                          ('inventory', True),
+                          ('stat-cache', False)]
+        # If a mandatory checkout file is not present, the branch does not have
+        # a functional checkout. Do not create a checkout in the converted
+        # branch.
+        for name, mandatory in checkout_files:
+            if mandatory and name not in bzrcontents:
+                has_checkout = False
+                break
+        else:
+            has_checkout = True
+        if not has_checkout:
+            ui.ui_factory.note('No working tree.')
+            # If some checkout files are there, we may as well get rid of them.
+            for name, mandatory in checkout_files:
+                if name in bzrcontents:
+                    self.bzrdir.transport.delete(name)
+        else:
+            from bzrlib.workingtree import WorkingTreeFormat3
+            self.step('Upgrading working tree')
+            self.bzrdir.transport.mkdir('checkout', mode=self.dir_mode)
+            self.make_lock('checkout')
+            self.put_format(
+                'checkout', WorkingTreeFormat3())
+            self.bzrdir.transport.delete_multi(
+                self.garbage_inventories, self.pb)
+            for entry in checkout_files:
+                self.move_entry('checkout', entry)
+            if last_revision is not None:
+                self.bzrdir.transport.put_bytes(
+                    'checkout/last-revision', last_revision)
+        self.bzrdir.transport.put_bytes(
+            'branch-format',
+            BzrDirMetaFormat1().get_format_string(),
+            mode=self.file_mode)
+        self.pb.finished()
+        return BzrDir.open(self.bzrdir.user_url)
+
+    def make_lock(self, name):
+        """Make a lock for the new control dir name."""
+        self.step('Make %s lock' % name)
+        ld = lockdir.LockDir(self.bzrdir.transport,
+                             '%s/lock' % name,
+                             file_modebits=self.file_mode,
+                             dir_modebits=self.dir_mode)
+        ld.create()
+
+    def move_entry(self, new_dir, entry):
+        """Move then entry name into new_dir."""
+        name = entry[0]
+        mandatory = entry[1]
+        self.step('Moving %s' % name)
+        try:
+            self.bzrdir.transport.move(name, '%s/%s' % (new_dir, name))
+        except errors.NoSuchFile:
+            if mandatory:
+                raise
+
+    def put_format(self, dirname, format):
+        self.bzrdir.transport.put_bytes('%s/format' % dirname,
+            format.get_format_string(),
+            self.file_mode)
+
+
+class BzrDirFormat4(BzrDirFormat):
+    """Bzr dir format 4.
+
+    This format is a combined format for working tree, branch and repository.
+    It has:
+     - Format 1 working trees [always]
+     - Format 4 branches [always]
+     - Format 4 repositories [always]
+
+    This format is deprecated: it indexes texts using a text it which is
+    removed in format 5; write support for this format has been removed.
+    """
+
+    _lock_class = lockable_files.TransportLock
+
+    def get_format_string(self):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar-NG branch, format 0.0.4\n"
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "All-in-one format 4"
+
+    def get_converter(self, format=None):
+        """See BzrDirFormat.get_converter()."""
+        # there is one and only one upgrade path here.
+        return ConvertBzrDir4To5()
+
+    def initialize_on_transport(self, transport):
+        """Format 4 branches cannot be created."""
+        raise errors.UninitializableFormat(self)
+
+    def is_supported(self):
+        """Format 4 is not supported.
+
+        It is not supported because the model changed from 4 to 5 and the
+        conversion logic is expensive - so doing it on the fly was not
+        feasible.
+        """
+        return False
+
+    def network_name(self):
+        return self.get_format_string()
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir4(transport, self)
+
+    def __return_repository_format(self):
+        """Circular import protection."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat4
+        return RepositoryFormat4()
+    repository_format = property(__return_repository_format)
+
+
+class BzrDirPreSplitOut(BzrDir):
+    """A common class for the all-in-one formats."""
+
+    def __init__(self, _transport, _format):
+        """See BzrDir.__init__."""
+        super(BzrDirPreSplitOut, self).__init__(_transport, _format)
+        self._control_files = lockable_files.LockableFiles(
+                                            self.get_branch_transport(None),
+                                            self._format._lock_file_name,
+                                            self._format._lock_class)
+
+    def break_lock(self):
+        """Pre-splitout bzrdirs do not suffer from stale locks."""
+        raise NotImplementedError(self.break_lock)
+
+    def cloning_metadir(self, require_stacking=False):
+        """Produce a metadir suitable for cloning with."""
+        if require_stacking:
+            return format_registry.make_bzrdir('1.6')
+        return self._format.__class__()
+
+    def clone(self, url, revision_id=None, force_new_repo=False,
+              preserve_stacking=False):
+        """See BzrDir.clone().
+
+        force_new_repo has no effect, since this family of formats always
+        require a new repository.
+        preserve_stacking has no effect, since no source branch using this
+        family of formats can be stacked, so there is no stacking to preserve.
+        """
+        self._make_tail(url)
+        result = self._format._initialize_for_clone(url)
+        self.open_repository().clone(result, revision_id=revision_id)
+        from_branch = self.open_branch()
+        from_branch.clone(result, revision_id=revision_id)
+        try:
+            tree = self.open_workingtree()
+        except errors.NotLocalUrl:
+            # make a new one, this format always has to have one.
+            result._init_workingtree()
+        else:
+            tree.clone(result)
+        return result
+
+    def create_branch(self, name=None, repository=None):
+        """See BzrDir.create_branch."""
+        if repository is not None:
+            raise NotImplementedError(
+                "create_branch(repository=<not None>) on %r" % (self,))
+        return self._format.get_branch_format().initialize(self, name=name)
+
+    def destroy_branch(self, name=None):
+        """See BzrDir.destroy_branch."""
+        raise errors.UnsupportedOperation(self.destroy_branch, self)
+
+    def create_repository(self, shared=False):
+        """See BzrDir.create_repository."""
+        if shared:
+            raise errors.IncompatibleFormat('shared repository', self._format)
+        return self.open_repository()
+
+    def destroy_repository(self):
+        """See BzrDir.destroy_repository."""
+        raise errors.UnsupportedOperation(self.destroy_repository, self)
+
+    def create_workingtree(self, revision_id=None, from_branch=None,
+                           accelerator_tree=None, hardlink=False):
+        """See BzrDir.create_workingtree."""
+        # The workingtree is sometimes created when the bzrdir is created,
+        # but not when cloning.
+
+        # this looks buggy but is not -really-
+        # because this format creates the workingtree when the bzrdir is
+        # created
+        # clone and sprout will have set the revision_id
+        # and that will have set it for us, its only
+        # specific uses of create_workingtree in isolation
+        # that can do wonky stuff here, and that only
+        # happens for creating checkouts, which cannot be
+        # done on this format anyway. So - acceptable wart.
+        if hardlink:
+            warning("can't support hardlinked working trees in %r"
+                % (self,))
+        try:
+            result = self.open_workingtree(recommend_upgrade=False)
+        except errors.NoSuchFile:
+            result = self._init_workingtree()
+        if revision_id is not None:
+            if revision_id == _mod_revision.NULL_REVISION:
+                result.set_parent_ids([])
+            else:
+                result.set_parent_ids([revision_id])
+        return result
+
+    def _init_workingtree(self):
+        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
+        try:
+            return WorkingTreeFormat2().initialize(self)
+        except errors.NotLocalUrl:
+            # Even though we can't access the working tree, we need to
+            # create its control files.
+            return WorkingTreeFormat2()._stub_initialize_on_transport(
+                self.transport, self._control_files._file_mode)
+
+    def destroy_workingtree(self):
+        """See BzrDir.destroy_workingtree."""
+        raise errors.UnsupportedOperation(self.destroy_workingtree, self)
+
+    def destroy_workingtree_metadata(self):
+        """See BzrDir.destroy_workingtree_metadata."""
+        raise errors.UnsupportedOperation(self.destroy_workingtree_metadata,
+                                          self)
+
+    def get_branch_transport(self, branch_format, name=None):
+        """See BzrDir.get_branch_transport()."""
+        if name is not None:
+            raise errors.NoColocatedBranchSupport(self)
+        if branch_format is None:
+            return self.transport
+        try:
+            branch_format.get_format_string()
+        except NotImplementedError:
+            return self.transport
+        raise errors.IncompatibleFormat(branch_format, self._format)
+
+    def get_repository_transport(self, repository_format):
+        """See BzrDir.get_repository_transport()."""
+        if repository_format is None:
+            return self.transport
+        try:
+            repository_format.get_format_string()
+        except NotImplementedError:
+            return self.transport
+        raise errors.IncompatibleFormat(repository_format, self._format)
+
+    def get_workingtree_transport(self, workingtree_format):
+        """See BzrDir.get_workingtree_transport()."""
+        if workingtree_format is None:
+            return self.transport
+        try:
+            workingtree_format.get_format_string()
+        except NotImplementedError:
+            return self.transport
+        raise errors.IncompatibleFormat(workingtree_format, self._format)
+
+    def needs_format_conversion(self, format=None):
+        """See BzrDir.needs_format_conversion()."""
+        # if the format is not the same as the system default,
+        # an upgrade is needed.
+        if format is None:
+            symbol_versioning.warn(symbol_versioning.deprecated_in((1, 13, 0))
+                % 'needs_format_conversion(format=None)')
+            format = BzrDirFormat.get_default_format()
+        return not isinstance(self._format, format.__class__)
+
+    def open_branch(self, name=None, unsupported=False,
+                    ignore_fallbacks=False):
+        """See BzrDir.open_branch."""
+        from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
+        format = BzrBranchFormat4()
+        self._check_supported(format, unsupported)
+        return format.open(self, name, _found=True)
+
+    def sprout(self, url, revision_id=None, force_new_repo=False,
+               possible_transports=None, accelerator_tree=None,
+               hardlink=False, stacked=False, create_tree_if_local=True,
+               source_branch=None):
+        """See BzrDir.sprout()."""
+        if source_branch is not None:
+            my_branch = self.open_branch()
+            if source_branch.base != my_branch.base:
+                raise AssertionError(
+                    "source branch %r is not within %r with branch %r" %
+                    (source_branch, self, my_branch))
+        if stacked:
+            raise errors.UnstackableBranchFormat(
+                self._format, self.root_transport.base)
+        if not create_tree_if_local:
+            raise errors.MustHaveWorkingTree(
+                self._format, self.root_transport.base)
+        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
+        self._make_tail(url)
+        result = self._format._initialize_for_clone(url)
+        try:
+            self.open_repository().clone(result, revision_id=revision_id)
+        except errors.NoRepositoryPresent:
+            pass
+        try:
+            self.open_branch().sprout(result, revision_id=revision_id)
+        except errors.NotBranchError:
+            pass
+
+        # we always want a working tree
+        WorkingTreeFormat2().initialize(result,
+                                        accelerator_tree=accelerator_tree,
+                                        hardlink=hardlink)
+        return result
+
+
+class BzrDir4(BzrDirPreSplitOut):
+    """A .bzr version 4 control object.
+
+    This is a deprecated format and may be removed after sept 2006.
+    """
+
+    def create_repository(self, shared=False):
+        """See BzrDir.create_repository."""
+        return self._format.repository_format.initialize(self, shared)
+
+    def needs_format_conversion(self, format=None):
+        """Format 4 dirs are always in need of conversion."""
+        if format is None:
+            symbol_versioning.warn(symbol_versioning.deprecated_in((1, 13, 0))
+                % 'needs_format_conversion(format=None)')
+        return True
+
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat4
+        return RepositoryFormat4().open(self, _found=True)
+
+
+
+
+class BzrDir5(BzrDirPreSplitOut):
+    """A .bzr version 5 control object.
+
+    This is a deprecated format and may be removed after sept 2006.
+    """
+
+    def has_workingtree(self):
+        """See BzrDir.has_workingtree."""
+        return True
+    
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat5
+        return RepositoryFormat5().open(self, _found=True)
+
+    def open_workingtree(self, _unsupported=False,
+            recommend_upgrade=True):
+        """See BzrDir.create_workingtree."""
+        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
+        wt_format = WorkingTreeFormat2()
+        # we don't warn here about upgrades; that ought to be handled for the
+        # bzrdir as a whole
+        return wt_format.open(self, _found=True)
+
+
+class BzrDir6(BzrDirPreSplitOut):
+    """A .bzr version 6 control object.
+
+    This is a deprecated format and may be removed after sept 2006.
+    """
+
+    def has_workingtree(self):
+        """See BzrDir.has_workingtree."""
+        return True
+
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.plugins.weave_fmt.repository import RepositoryFormat6
+        return RepositoryFormat6().open(self, _found=True)
+
+    def open_workingtree(self, _unsupported=False,
+        recommend_upgrade=True):
+        """See BzrDir.create_workingtree."""
+        # we don't warn here about upgrades; that ought to be handled for the
+        # bzrdir as a whole
+        from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
+        return WorkingTreeFormat2().open(self, _found=True)

=== modified file 'bzrlib/plugins/weave_fmt/repository.py'
--- bzrlib/plugins/weave_fmt/repository.py	2011-03-09 01:33:12 +0000
+++ bzrlib/plugins/weave_fmt/repository.py	2011-03-10 13:29:54 +0000
@@ -28,6 +28,7 @@
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
+    bzrdir_weave,
     xml5,
     graph as _mod_graph,
     ui,

=== added file 'bzrlib/plugins/weave_fmt/test_bzrdir.py'
--- bzrlib/plugins/weave_fmt/test_bzrdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/weave_fmt/test_bzrdir.py	2011-03-10 13:29:54 +0000
@@ -0,0 +1,512 @@
+# Copyright (C) 2006-2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for the weave-era BzrDir formats.
+
+For interface contract tests, see tests/per_bzr_dir.
+"""
+
+import sys
+
+from bzrlib import (
+    branch,
+    bzrdir,
+    repository,
+    upgrade,
+    workingtree,
+    )
+from bzrlib.osutils import (
+    lexists,
+    )
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
+from bzrlib.tests import (
+    TestCaseWithTransport,
+    )
+
+from bzrlib.bzrdir_weave import (
+    BzrDirFormat5,
+    BzrDirFormat6,
+    )
+
+
+class TestFormat5(TestCaseWithTransport):
+    """Tests specific to the version 5 bzrdir format."""
+
+    def test_same_lockfiles_between_tree_repo_branch(self):
+        # this checks that only a single lockfiles instance is created
+        # for format 5 objects
+        dir = BzrDirFormat5().initialize(self.get_url())
+        def check_dir_components_use_same_lock(dir):
+            ctrl_1 = dir.open_repository().control_files
+            ctrl_2 = dir.open_branch().control_files
+            ctrl_3 = dir.open_workingtree()._control_files
+            self.assertTrue(ctrl_1 is ctrl_2)
+            self.assertTrue(ctrl_2 is ctrl_3)
+        check_dir_components_use_same_lock(dir)
+        # and if we open it normally.
+        dir = bzrdir.BzrDir.open(self.get_url())
+        check_dir_components_use_same_lock(dir)
+
+    def test_can_convert(self):
+        # format 5 dirs are convertable
+        dir = BzrDirFormat5().initialize(self.get_url())
+        self.assertTrue(dir.can_convert_format())
+
+    def test_needs_conversion(self):
+        # format 5 dirs need a conversion if they are not the default,
+        # and they aren't
+        dir = BzrDirFormat5().initialize(self.get_url())
+        # don't need to convert it to itself
+        self.assertFalse(dir.needs_format_conversion(BzrDirFormat5()))
+        # do need to convert it to the current default
+        self.assertTrue(dir.needs_format_conversion(
+            bzrdir.BzrDirFormat.get_default_format()))
+
+
+class TestFormat6(TestCaseWithTransport):
+    """Tests specific to the version 6 bzrdir format."""
+
+    def test_same_lockfiles_between_tree_repo_branch(self):
+        # this checks that only a single lockfiles instance is created
+        # for format 6 objects
+        dir = BzrDirFormat6().initialize(self.get_url())
+        def check_dir_components_use_same_lock(dir):
+            ctrl_1 = dir.open_repository().control_files
+            ctrl_2 = dir.open_branch().control_files
+            ctrl_3 = dir.open_workingtree()._control_files
+            self.assertTrue(ctrl_1 is ctrl_2)
+            self.assertTrue(ctrl_2 is ctrl_3)
+        check_dir_components_use_same_lock(dir)
+        # and if we open it normally.
+        dir = bzrdir.BzrDir.open(self.get_url())
+        check_dir_components_use_same_lock(dir)
+
+    def test_can_convert(self):
+        # format 6 dirs are convertable
+        dir = BzrDirFormat6().initialize(self.get_url())
+        self.assertTrue(dir.can_convert_format())
+
+    def test_needs_conversion(self):
+        # format 6 dirs need an conversion if they are not the default.
+        dir = BzrDirFormat6().initialize(self.get_url())
+        self.assertTrue(dir.needs_format_conversion(
+            bzrdir.BzrDirFormat.get_default_format()))
+
+
+class TestBreakLockOldBranch(TestCaseWithTransport):
+
+    def test_break_lock_format_5_bzrdir(self):
+        # break lock on a format 5 bzrdir should just return
+        self.make_branch_and_tree('foo', format=BzrDirFormat5())
+        out, err = self.run_bzr('break-lock foo')
+        self.assertEqual('', out)
+        self.assertEqual('', err)
+
+
+_upgrade1_template = \
+     [
+     ('foo', 'new contents\n'),
+     ('.bzr/',),
+     ('.bzr/README',
+      'This is a Bazaar control directory.\n'
+      'Do not change any files in this directory.\n'
+      'See http://bazaar.canonical.com/ for more information about Bazaar.\n'),
+     ('.bzr/branch-format', 'Bazaar-NG branch, format 0.0.4\n'),
+     ('.bzr/revision-history',
+      'mbp@sourcefrog.net-20051004035611-176b16534b086b3c\n'
+      'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd\n'),
+     ('.bzr/merged-patches', ''),
+     ('.bzr/pending-merged-patches', ''),
+     ('.bzr/branch-name', ''),
+     ('.bzr/branch-lock', ''),
+     ('.bzr/pending-merges', ''),
+     ('.bzr/inventory',
+      '<inventory>\n'
+      '<entry file_id="foo-20051004035605-91e788d1875603ae" kind="file" name="foo" />\n'
+      '</inventory>\n'),
+     ('.bzr/stat-cache',
+      '### bzr hashcache v5\n'
+      'foo// be9f309239729f69a6309e970ef24941d31e042c 13 1128398176 1128398176 303464 770\n'),
+     ('.bzr/text-store/',),
+     ('.bzr/text-store/foo-20051004035611-1591048e9dc7c2d4.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\xcb\xcc\xcb,\xc9L\xccQH\xce\xcf+I\xcd+)\xe6\x02\x00\xdd\xcc\xf90\x11\x00\x00\x00'),
+     ('.bzr/text-store/foo-20051004035756-4081373d897c3453.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\xcbK-WH\xce\xcf+I\xcd+)\xe6\x02\x00g\xc3\xdf\xc9\r\x00\x00\x00'),
+     ('.bzr/inventory-store/',),
+     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xffm\x8f\xcd\n\xc20\x10\x84\xef>E\xc8\xbdt7?M\x02\xad\xaf"\xa1\x99`P[\xa8E\xacOo\x14\x05\x0f\xdef\xe1\xfbv\x98\xbeL7L\xeb\xbcl\xfb]_\xc3\xb2\x89\\\xce8\x944\xc8<\xcf\x8d"\xb2LdH\xdb\x8el\x13\x18\xce\xfb\xc4\xde\xd5SGHq*\xd3\x0b\xad\x8e\x14S\xbc\xe0\xadI\xb1\xe2\xbe\xfe}\xc2\xdc\xb0\rL\xc6#\xa4\xd1\x8d*\x99\x0f}=F\x1e$8G\x9d\xa0\x02\xa1rP9\x01c`FV\xda1qg\x98"\x02}\xa5\xf2\xa8\x95\xec\xa4h\xeb\x80\xf6g\xcd\x13\xb3\x01\xcc\x98\xda\x00\x00\x00'),
+     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xffm\x8f\xc1\n\xc20\x10D\xef~E\xc8\xbd\xb8\x9bM\x9a,\xb4\xfe\x8a\xc4f\x83Am\xa1\x16\xb1~\xbdQ\x14<x\x9b\x81y3LW\xc6\x9b\x8c\xcb4\xaf\xbbMW\xc5\xbc\xaa\\\xce\xb2/\xa9\xd7y\x9a\x1a\x03\xe0\x10\xc0\x02\xb9\x16\\\xc3(>\x84\x84\xc1WKQ\xb4:\x95\xf1\x15\xad\x8cVc\xbc\xc8\x1b\xd3j\x91\xfb\xf2\xaf\xa4r\x8d\x85\x80\xe4)\x05\xf6\x03YG\x9f\xf4\xf5\x18\xb1\xd7\x07\xe1L\xc0\x86\xd8\x1b\xce-\xc7\xb6:a\x0f\x92\x8de\x8b\x89P\xc0\x9a\xe1\x0b\x95G\x9d\xc4\xda\xb1\xad\x07\xb6?o\x9e\xb5\xff\xf0\xf9\xda\x00\x00\x00'),
+     ('.bzr/revision-store/',),
+     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\x9d\x8eKj\xc30\x14E\xe7^\x85\xd0 \xb3$\xefI\xd1\x8f\xd8\xa6\x1b(t\x07E?\xbb\x82H\n\xb2\x1ahW\xdfB1\x14:\xeb\xf4r\xee\xbdgl\xf1\x91\xb6T\x0b\xf15\xe7\xd4{l\x13}\xb6\xad\xa7B^j\xbd\x91\xc3\xad_\xb3\xbb?m\xf5\xbd\xf9\xb8\xb4\xba\x9eJ\xec\x87\xb5_)I\xe5\x11K\xaf\xed\xe35\x85\x89\xfe\xa5\x8e\x0c@ \xc0\x05\xb8\x90\x88GT\xd2\xa1\x14\xfc\xe2@K\xc7\xfd\xef\x85\xed\xcd\xe2D\x95\x8d\x1a\xa47<\x02c2\xb0 \xbc\xd0\x8ay\xa3\xbcp\x8a\x83\x12A3\xb7XJv\xef\x7f_\xf7\x94\xe3\xd6m\xbeO\x14\x91in4*<\x812\x88\xc60\xfc\x01>k\x89\x13\xe5\x12\x00\xe8<\x8c\xdf\x8d\xcd\xaeq\xb6!\x90\xa5\xd6\xf1\xbc\x07\xc3x\xde\x85\xe6\xe1\x0b\xc8\x8a\x98\x03T\x01\x00\x00'),
+     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\x9d\x90Kj\x031\x0c\x86\xf79\xc5\xe0Ev\xe9\xc8o\x9b\xcc\x84^\xa0\xd0\x1b\x14\xbf&5d\xec`\xbb\x81\xf6\xf45\x84\xa4\x81\xaeZ\xa1\x85\x84^\xdf\xaf\xa9\x84K\xac1\xa7\xc1\xe5u\x8d\xad\x852\xa3\x17SZL\xc3k\xce\xa7a{j\xfb\xd5\x9e\x9fk\xfe(.,%\x1f\x9fRh\xdbc\xdb\xa3!\xa6KH-\x97\xcf\xb7\xe8g\xf4\xbbkG\x008\x06`@\xb9\xe4bG(_\x88\x95\xde\xf9n\xca\xfb\xc7\r\xf5\xdd\xe0\x19\xa9\x85)\x81\xf5"\xbd\x04j\xb8\x02b\xa8W\\\x0b\xc9\x14\xf4\xbc\xbb\xd7\xd6H4\xdc\xb8\xff}\xba\xc55\xd4f\xd6\xf3\x8c0&\x8ajE\xa4x\xe2@\xa5\xa6\x9a\xf3k\xc3WNaFT\x00\x00:l\xa6>Q\xcd1\x1cjp9\xf9;\xc34\xde\n\x9b\xe9lJWT{t\',a\xf9\x0b\xae\xc0x\x87\xa5\xb0Xp\xca,(a\xa9{\xd0{}\xd4\x12\x04(\xc5\xbb$\xc5$V\xceaI\x19\x01\xa2\x1dh\xed\x82d\x8c.\xccr@\xc3\xd8Q\xc6\x1f\xaa\xf1\xb6\xe8\xb0\xf9\x06QR\r\xf9\xfc\x01\x00\x00')]
+
+
+_ghost_template = [
+    ( './foo',
+        'hello\n'
+    ),
+    ( './.bzr/', ),
+    ( './.bzr/README',
+      'This is a Bazaar control directory.\n'
+      'Do not change any files in this directory.\n'
+      'See http://bazaar.canonical.com/ for more information about Bazaar.\n'
+    ),
+    ( './.bzr/branch-format',
+        'Bazaar-NG branch, format 0.0.4\n'
+    ),
+    ( './.bzr/branch-lock',
+        ''
+    ),
+    ( './.bzr/branch-name',
+        ''
+    ),
+    ( './.bzr/inventory',
+        '<inventory>\n'
+        '<entry file_id="foo-20051004104918-0379cb7c76354cde" kind="file" name="foo" />\n'
+        '</inventory>\n'
+    ),
+    ( './.bzr/merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merges',
+        ''
+    ),
+    ( './.bzr/revision-history',
+        'mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\n'
+        'mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\n'
+    ),
+    ( './.bzr/stat-cache',
+        '### bzr hashcache v5\n'
+        'foo// f572d396fae9206628714fb2ce00f72e94f2258f 6 1128422956 1128422956 306900 770\n'
+    ),
+    ( './.bzr/text-store/', ),
+    ( './.bzr/text-store/foo-20051004104921-8de8118a71be45ba.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03foo-20051004104921-8de8118a71be45ba\x00\xcbH\xcd\xc9\xc9\xe7\x02\x00 0:6\x06\x00\x00\x00'
+    ),
+    ( './.bzr/inventory-store/', ),
+    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00m\x8f\xcb\n'
+        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
+        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
+    ),
+    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
+        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00m\x8f\xcb\n'
+        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
+        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
+    ),
+    ( './.bzr/revision-store/', ),
+    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00\x9d\x8eMj\xc30\x14\x84\xf7>\x85\xd0"\xbb$\xef\xc9\xb6,\x11\xdb\xf4\x02\x85\xde\xa0\xe8\xe7\xd9\x11\xc4R\x90\xd4@{\xfa\x06\x8a\xa1\xd0]\x97\x03\xdf\xcc|c\xa6G(!E\xe6\xd2\xb6\x85Z)O\xfc\xd5\xe4\x1a"{K\xe9\xc6\x0e\xb7z\xd9\xec\xfd\xa5\xa4\x8f\xech\xc9i=E\xaa\x87\xb5^8\x0b\xf1A\xb1\xa6\xfc\xf9\x1e\xfc\xc4\xffRG\x01\xd0#@\x87\xd0i\x81G\xa3\x95%!\x06\xe5}\x0bv\xb0\xbf\x17\xca\xd5\xe0\xc4-\xa0\xb1\x8b\xb6`\xc0I\xa4\xc5\xf4\x9el\xef\x95v [\x94\xcf\x8e\xd5\xcay\xe4l\xf7\xfe\xf7u\r'
+        '\x1b\x95j\xb6\xfb\xc4\x11\x85\xea\x84\xd0\x12O\x03t\x83D\xad\xc4\x0f\xf0\x95"M\xbc\x95\x00\xc0\xe7f|6\x8aYi^B.u<\xef\xb1\x19\xcf\xbb\xce\xdc|\x038=\xc7\xe6R\x01\x00\x00'
+    ),
+    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
+        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00\x9d\x90\xc1j\xc30\x0c\x86\xef}\n'
+        "\xe3Coie'\xb1c\x9a\x94\xbe\xc0`o0,[N\x03M\\\x1c\xafe{\xfae\x94n\x85\xc1`;Y\x88O\xd2\xff\xb9Mt\x19\xe6!N\xcc\xc5q\x1cr\xa6\xd4\xf1'\x9b\xf20\xb1\xe7\x18Ol}\xca\xbb\x11\xcf\x879\xbe&G!\xc5~3Q^\xf7y\xc7\xd90]h\xca1\xbd\xbd\x0c\xbe\xe3?\xa9B\x02\xd4\x02\xa0\x12P\x99R\x17\xce\xa0\xb6\x1a\x83s\x80(\xa5\x7f\xdc0\x1f\xad\xe88\x82\xb0\x18\x0c\x82\x05\xa7\x04\x05[{\xc2\xda7\xc6\x81*\x85B\x8dh\x1a\xe7\x05g\xf7\xdc\xff>\x9d\x87\x91\xe6l\xc7s\xc7\x85\x90M%\xa5\xd1z#\x85\xa8\x9b\x1a\xaa\xfa\x06\xbc\xc7\x89:^*\x00\xe0\xfbU\xbbL\xcc\xb6\xa7\xfdH\xa9'\x16\x03\xeb\x8fq\xce\xed\xf6\xde_\xb5g\x9b\x16\xa1y\xa9\xbe\x02&\n"
+        '\x7fJ+EaM\x83$\xa5n\xbc/a\x91~\xd0\xbd\xfd\x135\n'
+        '\xd0\x9a`\x0c*W\x1aR\xc1\x94du\x08(\t\xb0\x91\xdeZ\xa3\x9cU\x9cm\x7f\x8dr\x1d\x10Ot\xb8\xc6\xcf\xa7\x907|\xfb-\xb1\xbd\xd3\xfb\xd5\x07\xeeD\xee\x08*\x02\x00\x00'
+    ),
+]
+
+_upgrade_dir_template = [
+    ( './.bzr/', ),
+    ( './.bzr/README',
+      'This is a Bazaar control directory.\n'
+      'Do not change any files in this directory.\n'
+      'See http://bazaar.canonical.com/ for more information about Bazaar.\n'
+    ),
+    ( './.bzr/branch-format',
+        'Bazaar-NG branch, format 0.0.4\n'
+    ),
+    ( './.bzr/branch-lock',
+        ''
+    ),
+    ( './.bzr/branch-name',
+        ''
+    ),
+    ( './.bzr/inventory',
+        '<inventory>\n'
+        '<entry file_id="dir-20051005095101-da1441ea3fa6917a" kind="directory" name="dir" />\n'
+        '</inventory>\n'
+    ),
+    ( './.bzr/merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merges',
+        ''
+    ),
+    ( './.bzr/revision-history',
+        'robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e\n'
+    ),
+    ( './.bzr/stat-cache',
+        '### bzr hashcache v5\n'
+    ),
+    ( './.bzr/text-store/', ),
+    ( './.bzr/inventory-store/', ),
+    ( './.bzr/inventory-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
+        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xb3\xc9\xcc+K\xcd+\xc9/\xaa\xb4\xe3\xb2\x012\x8a*\x15\xd22sR\xe33Sl\x95R2\x8bt\x8d\x0c\x0cL\r'
+        "\x81\xd8\xc0\x12H\x19\xea\xa6$\x1a\x9a\x98\x18\xa6&\x1a\xa7%\x9aY\x1a\x9a'*)dg\xe6A\x94\xa6&\x83LQR\xc8K\xccM\x05\x0b()\xe8\x03\xcd\xd4G\xb2\x00\x00\xc2<\x94\xb1m\x00\x00\x00"
+    ),
+    ( './.bzr/revision-store/', ),
+    ( './.bzr/revision-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
+        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xa5OKj\xc30\x14\xdc\xfb\x14B\x8b\xec\x92<I\xd6\xc7\xc42\x85\xde\xa0\x17(\xb6\xf4\x9c\n'
+        'l\xa9H"\x90\x9c\xbe\xa6\xa9\xa1\x9b\xae\xbax\x0c\xcc\xe71\xd3g\xbc\x85\x12R$.\xadk\xa8\x15\xb3\xa5oi\xc2\\\xc9kZ\x96\x10\x0b9,\xf5\x92\xbf)\xf7\xf2\x83O\xe5\x14\xb1\x1e\xae\xf5BI\x887\x8c5\xe5\xfb{\xf0\x96\xfei>r\x00\xc9\xb6\x83n\x03sT\xa0\xe4<y\x83\xda\x1b\xc54\xfe~T>Ff\xe9\xcc:\xdd\x8e\xa6E\xc7@\xa2\x82I\xaaNL\xbas\\313)\x00\xb9\xe6\xe0(\xd9\x87\xfc\xb7A\r'
+        "+\x96:\xae\x9f\x962\xc6\x8d\x04i\x949\x01\x97R\xb7\x1d\x17O\xc3#E\xb4T(\x00\xa0C\xd3o\x892^q\x18\xbd'>\xe4\xfe\xbc\x13M\x7f\xde{\r"
+        '\xcd\x17\x85\xea\xba\x03l\x01\x00\x00'
+    ),
+    ( './dir/', ),
+]
+
+
+class TestUpgrade(TestCaseWithTransport):
+
+    def test_upgrade_v6_to_meta_no_workingtree(self):
+        # Some format6 branches do not have checkout files. Upgrading
+        # such a branch to metadir must not setup a working tree.
+        self.build_tree_contents(_upgrade1_template)
+        upgrade.upgrade('.', BzrDirFormat6())
+        t = self.get_transport('.')
+        t.delete_multi(['.bzr/pending-merges', '.bzr/inventory'])
+        self.assertFalse(t.has('.bzr/stat-cache'))
+        # XXX: upgrade fails if a backup.bzr is already present
+        # -- David Allouche 2006-08-11
+        t.delete_tree('backup.bzr.~1~')
+        # At this point, we have a format6 branch without checkout files.
+        upgrade.upgrade('.', bzrdir.BzrDirMetaFormat1())
+        # The upgrade should not have set up a working tree.
+        control = bzrdir.BzrDir.open('.')
+        self.assertFalse(control.has_workingtree())
+        # We have covered the scope of this test, we may as well check that
+        # upgrade has not eaten our data, even if it's a bit redundant with
+        # other tests.
+        self.failUnless(isinstance(control._format, bzrdir.BzrDirMetaFormat1))
+        b = control.open_branch()
+        self.assertEquals(b.revision_history(),
+           ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
+            'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
+
+    def test_upgrade_simple(self):
+        """Upgrade simple v0.0.4 format to latest format"""
+        eq = self.assertEquals
+        self.build_tree_contents(_upgrade1_template)
+        upgrade.upgrade(u'.')
+        control = bzrdir.BzrDir.open('.')
+        b = control.open_branch()
+        # tsk, peeking under the covers.
+        self.failUnless(
+            isinstance(
+                control._format,
+                bzrdir.BzrDirFormat.get_default_format().__class__))
+        rh = b.revision_history()
+        eq(rh,
+           ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
+            'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
+        rt = b.repository.revision_tree(rh[0])
+        foo_id = 'foo-20051004035605-91e788d1875603ae'
+        rt.lock_read()
+        try:
+            eq(rt.get_file_text(foo_id), 'initial contents\n')
+        finally:
+            rt.unlock()
+        rt = b.repository.revision_tree(rh[1])
+        rt.lock_read()
+        try:
+            eq(rt.get_file_text(foo_id), 'new contents\n')
+        finally:
+            rt.unlock()
+        # check a backup was made:
+        backup_dir = 'backup.bzr.~1~'
+        t = self.get_transport('.')
+        t.stat(backup_dir)
+        t.stat(backup_dir + '/README')
+        t.stat(backup_dir + '/branch-format')
+        t.stat(backup_dir + '/revision-history')
+        t.stat(backup_dir + '/merged-patches')
+        t.stat(backup_dir + '/pending-merged-patches')
+        t.stat(backup_dir + '/pending-merges')
+        t.stat(backup_dir + '/branch-name')
+        t.stat(backup_dir + '/branch-lock')
+        t.stat(backup_dir + '/inventory')
+        t.stat(backup_dir + '/stat-cache')
+        t.stat(backup_dir + '/text-store')
+        t.stat(backup_dir + '/text-store/foo-20051004035611-1591048e9dc7c2d4.gz')
+        t.stat(backup_dir + '/text-store/foo-20051004035756-4081373d897c3453.gz')
+        t.stat(backup_dir + '/inventory-store/')
+        t.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
+        t.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
+        t.stat(backup_dir + '/revision-store/')
+        t.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
+        t.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
+
+    def test_upgrade_with_ghosts(self):
+        """Upgrade v0.0.4 tree containing ghost references.
+
+        That is, some of the parents of revisions mentioned in the branch
+        aren't present in the branch's storage.
+
+        This shouldn't normally happen in branches created entirely in
+        bzr, but can happen in branches imported from baz and arch, or from
+        other systems, where the importer knows about a revision but not
+        its contents."""
+        eq = self.assertEquals
+        self.build_tree_contents(_ghost_template)
+        upgrade.upgrade(u'.')
+        b = branch.Branch.open(u'.')
+        revision_id = b.revision_history()[1]
+        rev = b.repository.get_revision(revision_id)
+        eq(len(rev.parent_ids), 2)
+        eq(rev.parent_ids[1], 'wibble@wobble-2')
+
+    def test_upgrade_makes_dir_weaves(self):
+        self.build_tree_contents(_upgrade_dir_template)
+        old_repodir = bzrdir.BzrDir.open_unsupported('.')
+        old_repo_format = old_repodir.open_repository()._format
+        upgrade.upgrade('.')
+        # this is the path to the literal file. As format changes
+        # occur it needs to be updated. FIXME: ask the store for the
+        # path.
+        repo = repository.Repository.open('.')
+        # it should have changed the format
+        self.assertNotEqual(old_repo_format.__class__, repo._format.__class__)
+        # and we should be able to read the names for the file id
+        # 'dir-20051005095101-da1441ea3fa6917a'
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        text_keys = repo.texts.keys()
+        dir_keys = [key for key in text_keys if key[0] ==
+                'dir-20051005095101-da1441ea3fa6917a']
+        self.assertNotEqual([], dir_keys)
+
+    def test_upgrade_to_meta_sets_workingtree_last_revision(self):
+        self.build_tree_contents(_upgrade_dir_template)
+        upgrade.upgrade('.', bzrdir.BzrDirMetaFormat1())
+        tree = workingtree.WorkingTree.open('.')
+        self.assertEqual([tree.branch.revision_history()[-1]],
+            tree.get_parent_ids())
+
+
+class SFTPBranchTest(TestCaseWithSFTPServer):
+    """Test some stuff when accessing a bzr Branch over sftp"""
+
+    def test_lock_file(self):
+        # old format branches use a special lock file on sftp.
+        b = self.make_branch('', format=BzrDirFormat6())
+        b = branch.Branch.open(self.get_url())
+        self.failUnlessExists('.bzr/')
+        self.failUnlessExists('.bzr/branch-format')
+        self.failUnlessExists('.bzr/branch-lock')
+
+        self.failIf(lexists('.bzr/branch-lock.write-lock'))
+        b.lock_write()
+        self.failUnlessExists('.bzr/branch-lock.write-lock')
+        b.unlock()
+        self.failIf(lexists('.bzr/branch-lock.write-lock'))
+
+
+class TestInfo(TestCaseWithTransport):
+
+    def test_info_locking_oslocks(self):
+        if sys.platform == "win32":
+            self.skip("don't use oslocks on win32 in unix manner")
+        # This test tests old (all-in-one, OS lock using) behaviour which
+        # simply cannot work on windows (and is indeed why we changed our
+        # design. As such, don't try to remove the thisFailsStrictLockCheck
+        # call here.
+        self.thisFailsStrictLockCheck()
+
+        tree = self.make_branch_and_tree('branch',
+                                         format=BzrDirFormat6())
+
+        # Test all permutations of locking the working tree, branch and repository
+        # XXX: Well not yet, as we can't query oslocks yet. Currently, it's
+        # implemented by raising NotImplementedError and get_physical_lock_status()
+        # always returns false. This makes bzr info hide the lock status.  (Olaf)
+        # W B R
+
+        # U U U
+        out, err = self.run_bzr('info -v branch')
+        self.assertEqualDiff(
+"""Standalone tree (format: weave)
+Location:
+  branch root: %s
+
+Format:
+       control: All-in-one format 6
+  working tree: Working tree format 2
+        branch: Branch format 4
+    repository: %s
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Repository:
+         0 revisions
+""" % ('branch', tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        # L L L
+        tree.lock_write()
+        out, err = self.run_bzr('info -v branch')
+        self.assertEqualDiff(
+"""Standalone tree (format: weave)
+Location:
+  branch root: %s
+
+Format:
+       control: All-in-one format 6
+  working tree: Working tree format 2
+        branch: Branch format 4
+    repository: %s
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Repository:
+         0 revisions
+""" % ('branch', tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        tree.unlock()
+
+

=== removed file 'bzrlib/plugins/weave_fmt/test_bzrdir.py'
--- bzrlib/plugins/weave_fmt/test_bzrdir.py	2011-02-18 22:40:35 +0000
+++ bzrlib/plugins/weave_fmt/test_bzrdir.py	1970-01-01 00:00:00 +0000
@@ -1,512 +0,0 @@
-# Copyright (C) 2006-2010 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-"""Tests for the weave-era BzrDir formats.
-
-For interface contract tests, see tests/per_bzr_dir.
-"""
-
-import sys
-
-from bzrlib import (
-    branch,
-    bzrdir,
-    repository,
-    upgrade,
-    workingtree,
-    )
-from bzrlib.osutils import (
-    lexists,
-    )
-from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
-from bzrlib.tests import (
-    TestCaseWithTransport,
-    )
-
-from bzrlib.plugins.weave_fmt.bzrdir import (
-    BzrDirFormat5,
-    BzrDirFormat6,
-    )
-
-
-class TestFormat5(TestCaseWithTransport):
-    """Tests specific to the version 5 bzrdir format."""
-
-    def test_same_lockfiles_between_tree_repo_branch(self):
-        # this checks that only a single lockfiles instance is created
-        # for format 5 objects
-        dir = BzrDirFormat5().initialize(self.get_url())
-        def check_dir_components_use_same_lock(dir):
-            ctrl_1 = dir.open_repository().control_files
-            ctrl_2 = dir.open_branch().control_files
-            ctrl_3 = dir.open_workingtree()._control_files
-            self.assertTrue(ctrl_1 is ctrl_2)
-            self.assertTrue(ctrl_2 is ctrl_3)
-        check_dir_components_use_same_lock(dir)
-        # and if we open it normally.
-        dir = bzrdir.BzrDir.open(self.get_url())
-        check_dir_components_use_same_lock(dir)
-
-    def test_can_convert(self):
-        # format 5 dirs are convertable
-        dir = BzrDirFormat5().initialize(self.get_url())
-        self.assertTrue(dir.can_convert_format())
-
-    def test_needs_conversion(self):
-        # format 5 dirs need a conversion if they are not the default,
-        # and they aren't
-        dir = BzrDirFormat5().initialize(self.get_url())
-        # don't need to convert it to itself
-        self.assertFalse(dir.needs_format_conversion(BzrDirFormat5()))
-        # do need to convert it to the current default
-        self.assertTrue(dir.needs_format_conversion(
-            bzrdir.BzrDirFormat.get_default_format()))
-
-
-class TestFormat6(TestCaseWithTransport):
-    """Tests specific to the version 6 bzrdir format."""
-
-    def test_same_lockfiles_between_tree_repo_branch(self):
-        # this checks that only a single lockfiles instance is created
-        # for format 6 objects
-        dir = BzrDirFormat6().initialize(self.get_url())
-        def check_dir_components_use_same_lock(dir):
-            ctrl_1 = dir.open_repository().control_files
-            ctrl_2 = dir.open_branch().control_files
-            ctrl_3 = dir.open_workingtree()._control_files
-            self.assertTrue(ctrl_1 is ctrl_2)
-            self.assertTrue(ctrl_2 is ctrl_3)
-        check_dir_components_use_same_lock(dir)
-        # and if we open it normally.
-        dir = bzrdir.BzrDir.open(self.get_url())
-        check_dir_components_use_same_lock(dir)
-
-    def test_can_convert(self):
-        # format 6 dirs are convertable
-        dir = BzrDirFormat6().initialize(self.get_url())
-        self.assertTrue(dir.can_convert_format())
-
-    def test_needs_conversion(self):
-        # format 6 dirs need an conversion if they are not the default.
-        dir = BzrDirFormat6().initialize(self.get_url())
-        self.assertTrue(dir.needs_format_conversion(
-            bzrdir.BzrDirFormat.get_default_format()))
-
-
-class TestBreakLockOldBranch(TestCaseWithTransport):
-
-    def test_break_lock_format_5_bzrdir(self):
-        # break lock on a format 5 bzrdir should just return
-        self.make_branch_and_tree('foo', format=BzrDirFormat5())
-        out, err = self.run_bzr('break-lock foo')
-        self.assertEqual('', out)
-        self.assertEqual('', err)
-
-
-_upgrade1_template = \
-     [
-     ('foo', 'new contents\n'),
-     ('.bzr/',),
-     ('.bzr/README',
-      'This is a Bazaar control directory.\n'
-      'Do not change any files in this directory.\n'
-      'See http://bazaar.canonical.com/ for more information about Bazaar.\n'),
-     ('.bzr/branch-format', 'Bazaar-NG branch, format 0.0.4\n'),
-     ('.bzr/revision-history',
-      'mbp@sourcefrog.net-20051004035611-176b16534b086b3c\n'
-      'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd\n'),
-     ('.bzr/merged-patches', ''),
-     ('.bzr/pending-merged-patches', ''),
-     ('.bzr/branch-name', ''),
-     ('.bzr/branch-lock', ''),
-     ('.bzr/pending-merges', ''),
-     ('.bzr/inventory',
-      '<inventory>\n'
-      '<entry file_id="foo-20051004035605-91e788d1875603ae" kind="file" name="foo" />\n'
-      '</inventory>\n'),
-     ('.bzr/stat-cache',
-      '### bzr hashcache v5\n'
-      'foo// be9f309239729f69a6309e970ef24941d31e042c 13 1128398176 1128398176 303464 770\n'),
-     ('.bzr/text-store/',),
-     ('.bzr/text-store/foo-20051004035611-1591048e9dc7c2d4.gz',
-      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\xcb\xcc\xcb,\xc9L\xccQH\xce\xcf+I\xcd+)\xe6\x02\x00\xdd\xcc\xf90\x11\x00\x00\x00'),
-     ('.bzr/text-store/foo-20051004035756-4081373d897c3453.gz',
-      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\xcbK-WH\xce\xcf+I\xcd+)\xe6\x02\x00g\xc3\xdf\xc9\r\x00\x00\x00'),
-     ('.bzr/inventory-store/',),
-     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
-      '\x1f\x8b\x08\x00[\xfdAC\x02\xffm\x8f\xcd\n\xc20\x10\x84\xef>E\xc8\xbdt7?M\x02\xad\xaf"\xa1\x99`P[\xa8E\xacOo\x14\x05\x0f\xdef\xe1\xfbv\x98\xbeL7L\xeb\xbcl\xfb]_\xc3\xb2\x89\\\xce8\x944\xc8<\xcf\x8d"\xb2LdH\xdb\x8el\x13\x18\xce\xfb\xc4\xde\xd5SGHq*\xd3\x0b\xad\x8e\x14S\xbc\xe0\xadI\xb1\xe2\xbe\xfe}\xc2\xdc\xb0\rL\xc6#\xa4\xd1\x8d*\x99\x0f}=F\x1e$8G\x9d\xa0\x02\xa1rP9\x01c`FV\xda1qg\x98"\x02}\xa5\xf2\xa8\x95\xec\xa4h\xeb\x80\xf6g\xcd\x13\xb3\x01\xcc\x98\xda\x00\x00\x00'),
-     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
-      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xffm\x8f\xc1\n\xc20\x10D\xef~E\xc8\xbd\xb8\x9bM\x9a,\xb4\xfe\x8a\xc4f\x83Am\xa1\x16\xb1~\xbdQ\x14<x\x9b\x81y3LW\xc6\x9b\x8c\xcb4\xaf\xbbMW\xc5\xbc\xaa\\\xce\xb2/\xa9\xd7y\x9a\x1a\x03\xe0\x10\xc0\x02\xb9\x16\\\xc3(>\x84\x84\xc1WKQ\xb4:\x95\xf1\x15\xad\x8cVc\xbc\xc8\x1b\xd3j\x91\xfb\xf2\xaf\xa4r\x8d\x85\x80\xe4)\x05\xf6\x03YG\x9f\xf4\xf5\x18\xb1\xd7\x07\xe1L\xc0\x86\xd8\x1b\xce-\xc7\xb6:a\x0f\x92\x8de\x8b\x89P\xc0\x9a\xe1\x0b\x95G\x9d\xc4\xda\xb1\xad\x07\xb6?o\x9e\xb5\xff\xf0\xf9\xda\x00\x00\x00'),
-     ('.bzr/revision-store/',),
-     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
-      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\x9d\x8eKj\xc30\x14E\xe7^\x85\xd0 \xb3$\xefI\xd1\x8f\xd8\xa6\x1b(t\x07E?\xbb\x82H\n\xb2\x1ahW\xdfB1\x14:\xeb\xf4r\xee\xbdgl\xf1\x91\xb6T\x0b\xf15\xe7\xd4{l\x13}\xb6\xad\xa7B^j\xbd\x91\xc3\xad_\xb3\xbb?m\xf5\xbd\xf9\xb8\xb4\xba\x9eJ\xec\x87\xb5_)I\xe5\x11K\xaf\xed\xe35\x85\x89\xfe\xa5\x8e\x0c@ \xc0\x05\xb8\x90\x88GT\xd2\xa1\x14\xfc\xe2@K\xc7\xfd\xef\x85\xed\xcd\xe2D\x95\x8d\x1a\xa47<\x02c2\xb0 \xbc\xd0\x8ay\xa3\xbcp\x8a\x83\x12A3\xb7XJv\xef\x7f_\xf7\x94\xe3\xd6m\xbeO\x14\x91in4*<\x812\x88\xc60\xfc\x01>k\x89\x13\xe5\x12\x00\xe8<\x8c\xdf\x8d\xcd\xaeq\xb6!\x90\xa5\xd6\xf1\xbc\x07\xc3x\xde\x85\xe6\xe1\x0b\xc8\x8a\x98\x03T\x01\x00\x00'),
-     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
-      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\x9d\x90Kj\x031\x0c\x86\xf79\xc5\xe0Ev\xe9\xc8o\x9b\xcc\x84^\xa0\xd0\x1b\x14\xbf&5d\xec`\xbb\x81\xf6\xf45\x84\xa4\x81\xaeZ\xa1\x85\x84^\xdf\xaf\xa9\x84K\xac1\xa7\xc1\xe5u\x8d\xad\x852\xa3\x17SZL\xc3k\xce\xa7a{j\xfb\xd5\x9e\x9fk\xfe(.,%\x1f\x9fRh\xdbc\xdb\xa3!\xa6KH-\x97\xcf\xb7\xe8g\xf4\xbbkG\x008\x06`@\xb9\xe4bG(_\x88\x95\xde\xf9n\xca\xfb\xc7\r\xf5\xdd\xe0\x19\xa9\x85)\x81\xf5"\xbd\x04j\xb8\x02b\xa8W\\\x0b\xc9\x14\xf4\xbc\xbb\xd7\xd6H4\xdc\xb8\xff}\xba\xc55\xd4f\xd6\xf3\x8c0&\x8ajE\xa4x\xe2@\xa5\xa6\x9a\xf3k\xc3WNaFT\x00\x00:l\xa6>Q\xcd1\x1cjp9\xf9;\xc34\xde\n\x9b\xe9lJWT{t\',a\xf9\x0b\xae\xc0x\x87\xa5\xb0Xp\xca,(a\xa9{\xd0{}\xd4\x12\x04(\xc5\xbb$\xc5$V\xceaI\x19\x01\xa2\x1dh\xed\x82d\x8c.\xccr@\xc3\xd8Q\xc6\x1f\xaa\xf1\xb6\xe8\xb0\xf9\x06QR\r\xf9\xfc\x01\x00\x00')]
-
-
-_ghost_template = [
-    ( './foo',
-        'hello\n'
-    ),
-    ( './.bzr/', ),
-    ( './.bzr/README',
-      'This is a Bazaar control directory.\n'
-      'Do not change any files in this directory.\n'
-      'See http://bazaar.canonical.com/ for more information about Bazaar.\n'
-    ),
-    ( './.bzr/branch-format',
-        'Bazaar-NG branch, format 0.0.4\n'
-    ),
-    ( './.bzr/branch-lock',
-        ''
-    ),
-    ( './.bzr/branch-name',
-        ''
-    ),
-    ( './.bzr/inventory',
-        '<inventory>\n'
-        '<entry file_id="foo-20051004104918-0379cb7c76354cde" kind="file" name="foo" />\n'
-        '</inventory>\n'
-    ),
-    ( './.bzr/merged-patches',
-        ''
-    ),
-    ( './.bzr/pending-merged-patches',
-        ''
-    ),
-    ( './.bzr/pending-merges',
-        ''
-    ),
-    ( './.bzr/revision-history',
-        'mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\n'
-        'mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\n'
-    ),
-    ( './.bzr/stat-cache',
-        '### bzr hashcache v5\n'
-        'foo// f572d396fae9206628714fb2ce00f72e94f2258f 6 1128422956 1128422956 306900 770\n'
-    ),
-    ( './.bzr/text-store/', ),
-    ( './.bzr/text-store/foo-20051004104921-8de8118a71be45ba.gz',
-        '\x1f\x8b\x08\x081^BC\x00\x03foo-20051004104921-8de8118a71be45ba\x00\xcbH\xcd\xc9\xc9\xe7\x02\x00 0:6\x06\x00\x00\x00'
-    ),
-    ( './.bzr/inventory-store/', ),
-    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
-        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00m\x8f\xcb\n'
-        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
-        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
-    ),
-    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
-        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00m\x8f\xcb\n'
-        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
-        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
-    ),
-    ( './.bzr/revision-store/', ),
-    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
-        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00\x9d\x8eMj\xc30\x14\x84\xf7>\x85\xd0"\xbb$\xef\xc9\xb6,\x11\xdb\xf4\x02\x85\xde\xa0\xe8\xe7\xd9\x11\xc4R\x90\xd4@{\xfa\x06\x8a\xa1\xd0]\x97\x03\xdf\xcc|c\xa6G(!E\xe6\xd2\xb6\x85Z)O\xfc\xd5\xe4\x1a"{K\xe9\xc6\x0e\xb7z\xd9\xec\xfd\xa5\xa4\x8f\xech\xc9i=E\xaa\x87\xb5^8\x0b\xf1A\xb1\xa6\xfc\xf9\x1e\xfc\xc4\xffRG\x01\xd0#@\x87\xd0i\x81G\xa3\x95%!\x06\xe5}\x0bv\xb0\xbf\x17\xca\xd5\xe0\xc4-\xa0\xb1\x8b\xb6`\xc0I\xa4\xc5\xf4\x9el\xef\x95v [\x94\xcf\x8e\xd5\xcay\xe4l\xf7\xfe\xf7u\r'
-        '\x1b\x95j\xb6\xfb\xc4\x11\x85\xea\x84\xd0\x12O\x03t\x83D\xad\xc4\x0f\xf0\x95"M\xbc\x95\x00\xc0\xe7f|6\x8aYi^B.u<\xef\xb1\x19\xcf\xbb\xce\xdc|\x038=\xc7\xe6R\x01\x00\x00'
-    ),
-    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
-        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00\x9d\x90\xc1j\xc30\x0c\x86\xef}\n'
-        "\xe3Coie'\xb1c\x9a\x94\xbe\xc0`o0,[N\x03M\\\x1c\xafe{\xfae\x94n\x85\xc1`;Y\x88O\xd2\xff\xb9Mt\x19\xe6!N\xcc\xc5q\x1cr\xa6\xd4\xf1'\x9b\xf20\xb1\xe7\x18Ol}\xca\xbb\x11\xcf\x879\xbe&G!\xc5~3Q^\xf7y\xc7\xd90]h\xca1\xbd\xbd\x0c\xbe\xe3?\xa9B\x02\xd4\x02\xa0\x12P\x99R\x17\xce\xa0\xb6\x1a\x83s\x80(\xa5\x7f\xdc0\x1f\xad\xe88\x82\xb0\x18\x0c\x82\x05\xa7\x04\x05[{\xc2\xda7\xc6\x81*\x85B\x8dh\x1a\xe7\x05g\xf7\xdc\xff>\x9d\x87\x91\xe6l\xc7s\xc7\x85\x90M%\xa5\xd1z#\x85\xa8\x9b\x1a\xaa\xfa\x06\xbc\xc7\x89:^*\x00\xe0\xfbU\xbbL\xcc\xb6\xa7\xfdH\xa9'\x16\x03\xeb\x8fq\xce\xed\xf6\xde_\xb5g\x9b\x16\xa1y\xa9\xbe\x02&\n"
-        '\x7fJ+EaM\x83$\xa5n\xbc/a\x91~\xd0\xbd\xfd\x135\n'
-        '\xd0\x9a`\x0c*W\x1aR\xc1\x94du\x08(\t\xb0\x91\xdeZ\xa3\x9cU\x9cm\x7f\x8dr\x1d\x10Ot\xb8\xc6\xcf\xa7\x907|\xfb-\xb1\xbd\xd3\xfb\xd5\x07\xeeD\xee\x08*\x02\x00\x00'
-    ),
-]
-
-_upgrade_dir_template = [
-    ( './.bzr/', ),
-    ( './.bzr/README',
-      'This is a Bazaar control directory.\n'
-      'Do not change any files in this directory.\n'
-      'See http://bazaar.canonical.com/ for more information about Bazaar.\n'
-    ),
-    ( './.bzr/branch-format',
-        'Bazaar-NG branch, format 0.0.4\n'
-    ),
-    ( './.bzr/branch-lock',
-        ''
-    ),
-    ( './.bzr/branch-name',
-        ''
-    ),
-    ( './.bzr/inventory',
-        '<inventory>\n'
-        '<entry file_id="dir-20051005095101-da1441ea3fa6917a" kind="directory" name="dir" />\n'
-        '</inventory>\n'
-    ),
-    ( './.bzr/merged-patches',
-        ''
-    ),
-    ( './.bzr/pending-merged-patches',
-        ''
-    ),
-    ( './.bzr/pending-merges',
-        ''
-    ),
-    ( './.bzr/revision-history',
-        'robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e\n'
-    ),
-    ( './.bzr/stat-cache',
-        '### bzr hashcache v5\n'
-    ),
-    ( './.bzr/text-store/', ),
-    ( './.bzr/inventory-store/', ),
-    ( './.bzr/inventory-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
-        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xb3\xc9\xcc+K\xcd+\xc9/\xaa\xb4\xe3\xb2\x012\x8a*\x15\xd22sR\xe33Sl\x95R2\x8bt\x8d\x0c\x0cL\r'
-        "\x81\xd8\xc0\x12H\x19\xea\xa6$\x1a\x9a\x98\x18\xa6&\x1a\xa7%\x9aY\x1a\x9a'*)dg\xe6A\x94\xa6&\x83LQR\xc8K\xccM\x05\x0b()\xe8\x03\xcd\xd4G\xb2\x00\x00\xc2<\x94\xb1m\x00\x00\x00"
-    ),
-    ( './.bzr/revision-store/', ),
-    ( './.bzr/revision-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
-        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xa5OKj\xc30\x14\xdc\xfb\x14B\x8b\xec\x92<I\xd6\xc7\xc42\x85\xde\xa0\x17(\xb6\xf4\x9c\n'
-        'l\xa9H"\x90\x9c\xbe\xa6\xa9\xa1\x9b\xae\xbax\x0c\xcc\xe71\xd3g\xbc\x85\x12R$.\xadk\xa8\x15\xb3\xa5oi\xc2\\\xc9kZ\x96\x10\x0b9,\xf5\x92\xbf)\xf7\xf2\x83O\xe5\x14\xb1\x1e\xae\xf5BI\x887\x8c5\xe5\xfb{\xf0\x96\xfei>r\x00\xc9\xb6\x83n\x03sT\xa0\xe4<y\x83\xda\x1b\xc54\xfe~T>Ff\xe9\xcc:\xdd\x8e\xa6E\xc7@\xa2\x82I\xaaNL\xbas\\313)\x00\xb9\xe6\xe0(\xd9\x87\xfc\xb7A\r'
-        "+\x96:\xae\x9f\x962\xc6\x8d\x04i\x949\x01\x97R\xb7\x1d\x17O\xc3#E\xb4T(\x00\xa0C\xd3o\x892^q\x18\xbd'>\xe4\xfe\xbc\x13M\x7f\xde{\r"
-        '\xcd\x17\x85\xea\xba\x03l\x01\x00\x00'
-    ),
-    ( './dir/', ),
-]
-
-
-class TestUpgrade(TestCaseWithTransport):
-
-    def test_upgrade_v6_to_meta_no_workingtree(self):
-        # Some format6 branches do not have checkout files. Upgrading
-        # such a branch to metadir must not setup a working tree.
-        self.build_tree_contents(_upgrade1_template)
-        upgrade.upgrade('.', BzrDirFormat6())
-        t = self.get_transport('.')
-        t.delete_multi(['.bzr/pending-merges', '.bzr/inventory'])
-        self.assertFalse(t.has('.bzr/stat-cache'))
-        # XXX: upgrade fails if a backup.bzr is already present
-        # -- David Allouche 2006-08-11
-        t.delete_tree('backup.bzr.~1~')
-        # At this point, we have a format6 branch without checkout files.
-        upgrade.upgrade('.', bzrdir.BzrDirMetaFormat1())
-        # The upgrade should not have set up a working tree.
-        control = bzrdir.BzrDir.open('.')
-        self.assertFalse(control.has_workingtree())
-        # We have covered the scope of this test, we may as well check that
-        # upgrade has not eaten our data, even if it's a bit redundant with
-        # other tests.
-        self.failUnless(isinstance(control._format, bzrdir.BzrDirMetaFormat1))
-        b = control.open_branch()
-        self.assertEquals(b.revision_history(),
-           ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
-            'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
-
-    def test_upgrade_simple(self):
-        """Upgrade simple v0.0.4 format to latest format"""
-        eq = self.assertEquals
-        self.build_tree_contents(_upgrade1_template)
-        upgrade.upgrade(u'.')
-        control = bzrdir.BzrDir.open('.')
-        b = control.open_branch()
-        # tsk, peeking under the covers.
-        self.failUnless(
-            isinstance(
-                control._format,
-                bzrdir.BzrDirFormat.get_default_format().__class__))
-        rh = b.revision_history()
-        eq(rh,
-           ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
-            'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
-        rt = b.repository.revision_tree(rh[0])
-        foo_id = 'foo-20051004035605-91e788d1875603ae'
-        rt.lock_read()
-        try:
-            eq(rt.get_file_text(foo_id), 'initial contents\n')
-        finally:
-            rt.unlock()
-        rt = b.repository.revision_tree(rh[1])
-        rt.lock_read()
-        try:
-            eq(rt.get_file_text(foo_id), 'new contents\n')
-        finally:
-            rt.unlock()
-        # check a backup was made:
-        backup_dir = 'backup.bzr.~1~'
-        t = self.get_transport('.')
-        t.stat(backup_dir)
-        t.stat(backup_dir + '/README')
-        t.stat(backup_dir + '/branch-format')
-        t.stat(backup_dir + '/revision-history')
-        t.stat(backup_dir + '/merged-patches')
-        t.stat(backup_dir + '/pending-merged-patches')
-        t.stat(backup_dir + '/pending-merges')
-        t.stat(backup_dir + '/branch-name')
-        t.stat(backup_dir + '/branch-lock')
-        t.stat(backup_dir + '/inventory')
-        t.stat(backup_dir + '/stat-cache')
-        t.stat(backup_dir + '/text-store')
-        t.stat(backup_dir + '/text-store/foo-20051004035611-1591048e9dc7c2d4.gz')
-        t.stat(backup_dir + '/text-store/foo-20051004035756-4081373d897c3453.gz')
-        t.stat(backup_dir + '/inventory-store/')
-        t.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
-        t.stat(backup_dir + '/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
-        t.stat(backup_dir + '/revision-store/')
-        t.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz')
-        t.stat(backup_dir + '/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz')
-
-    def test_upgrade_with_ghosts(self):
-        """Upgrade v0.0.4 tree containing ghost references.
-
-        That is, some of the parents of revisions mentioned in the branch
-        aren't present in the branch's storage.
-
-        This shouldn't normally happen in branches created entirely in
-        bzr, but can happen in branches imported from baz and arch, or from
-        other systems, where the importer knows about a revision but not
-        its contents."""
-        eq = self.assertEquals
-        self.build_tree_contents(_ghost_template)
-        upgrade.upgrade(u'.')
-        b = branch.Branch.open(u'.')
-        revision_id = b.revision_history()[1]
-        rev = b.repository.get_revision(revision_id)
-        eq(len(rev.parent_ids), 2)
-        eq(rev.parent_ids[1], 'wibble@wobble-2')
-
-    def test_upgrade_makes_dir_weaves(self):
-        self.build_tree_contents(_upgrade_dir_template)
-        old_repodir = bzrdir.BzrDir.open_unsupported('.')
-        old_repo_format = old_repodir.open_repository()._format
-        upgrade.upgrade('.')
-        # this is the path to the literal file. As format changes
-        # occur it needs to be updated. FIXME: ask the store for the
-        # path.
-        repo = repository.Repository.open('.')
-        # it should have changed the format
-        self.assertNotEqual(old_repo_format.__class__, repo._format.__class__)
-        # and we should be able to read the names for the file id
-        # 'dir-20051005095101-da1441ea3fa6917a'
-        repo.lock_read()
-        self.addCleanup(repo.unlock)
-        text_keys = repo.texts.keys()
-        dir_keys = [key for key in text_keys if key[0] ==
-                'dir-20051005095101-da1441ea3fa6917a']
-        self.assertNotEqual([], dir_keys)
-
-    def test_upgrade_to_meta_sets_workingtree_last_revision(self):
-        self.build_tree_contents(_upgrade_dir_template)
-        upgrade.upgrade('.', bzrdir.BzrDirMetaFormat1())
-        tree = workingtree.WorkingTree.open('.')
-        self.assertEqual([tree.branch.revision_history()[-1]],
-            tree.get_parent_ids())
-
-
-class SFTPBranchTest(TestCaseWithSFTPServer):
-    """Test some stuff when accessing a bzr Branch over sftp"""
-
-    def test_lock_file(self):
-        # old format branches use a special lock file on sftp.
-        b = self.make_branch('', format=BzrDirFormat6())
-        b = branch.Branch.open(self.get_url())
-        self.failUnlessExists('.bzr/')
-        self.failUnlessExists('.bzr/branch-format')
-        self.failUnlessExists('.bzr/branch-lock')
-
-        self.failIf(lexists('.bzr/branch-lock.write-lock'))
-        b.lock_write()
-        self.failUnlessExists('.bzr/branch-lock.write-lock')
-        b.unlock()
-        self.failIf(lexists('.bzr/branch-lock.write-lock'))
-
-
-class TestInfo(TestCaseWithTransport):
-
-    def test_info_locking_oslocks(self):
-        if sys.platform == "win32":
-            self.skip("don't use oslocks on win32 in unix manner")
-        # This test tests old (all-in-one, OS lock using) behaviour which
-        # simply cannot work on windows (and is indeed why we changed our
-        # design. As such, don't try to remove the thisFailsStrictLockCheck
-        # call here.
-        self.thisFailsStrictLockCheck()
-
-        tree = self.make_branch_and_tree('branch',
-                                         format=BzrDirFormat6())
-
-        # Test all permutations of locking the working tree, branch and repository
-        # XXX: Well not yet, as we can't query oslocks yet. Currently, it's
-        # implemented by raising NotImplementedError and get_physical_lock_status()
-        # always returns false. This makes bzr info hide the lock status.  (Olaf)
-        # W B R
-
-        # U U U
-        out, err = self.run_bzr('info -v branch')
-        self.assertEqualDiff(
-"""Standalone tree (format: weave)
-Location:
-  branch root: %s
-
-Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: %s
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Repository:
-         0 revisions
-""" % ('branch', tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        # L L L
-        tree.lock_write()
-        out, err = self.run_bzr('info -v branch')
-        self.assertEqualDiff(
-"""Standalone tree (format: weave)
-Location:
-  branch root: %s
-
-Format:
-       control: All-in-one format 6
-  working tree: Working tree format 2
-        branch: Branch format 4
-    repository: %s
-
-In the working tree:
-         0 unchanged
-         0 modified
-         0 added
-         0 removed
-         0 renamed
-         0 unknown
-         0 ignored
-         0 versioned subdirectories
-
-Branch history:
-         0 revisions
-
-Repository:
-         0 revisions
-""" % ('branch', tree.branch.repository._format.get_format_description(),
-       ), out)
-        self.assertEqual('', err)
-        tree.unlock()
-
-

=== modified file 'bzrlib/registry.py'
--- bzrlib/registry.py	2011-03-03 02:28:57 +0000
+++ bzrlib/registry.py	2011-03-10 12:59:19 +0000
@@ -43,6 +43,13 @@
         """Get the object that was saved at creation time"""
         return self._obj
 
+    def __eq__(self, other):
+        if other == self.get_obj():
+            return True
+        if isinstance(other, self.__class__):
+            return (self.get_obj() == other.get_obj())
+        return False
+
 
 class _LazyObjectGetter(_ObjectGetter):
     """Keep a record of a possible object.
@@ -74,6 +81,12 @@
             self._imported = True
         return super(_LazyObjectGetter, self).get_obj()
 
+    def __eq__(self, other):
+        if isinstance(other, self.__class__):
+            return (self._member_name == other._member_name and
+                    self._module_name == other._module_name)
+        return super(_LazyObjectGetter, self).__eq__(other)
+
     def __repr__(self):
         return "<%s.%s object at %x, module=%r attribute=%r imported=%r>" % (
             self.__class__.__module__, self.__class__.__name__, id(self),

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2011-02-24 16:45:27 +0000
+++ bzrlib/tests/test_bzrdir.py	2011-03-10 13:29:54 +0000
@@ -318,6 +318,22 @@
         # now open_downlevel should fail too.
         self.assertRaises(UnknownFormatError, bzrdir.BzrDir.open_unsupported, url)
 
+    def test_register_unregister_format_lazy(self):
+        # register a format for it.
+        bzrdir.BzrDirFormat.register_lazy_format("Test format 1",
+            "bzrlib.tests.test_bzrdir", "BzrDirFormatTest1")
+        # It should be registered with the prober:
+        self.assertIsInstance(bzrdir.BzrProber.formats.get("Test format 1"),
+            BzrDirFormatTest1)
+        self.assertTrue(BzrDirFormatTest1 in
+            controldir.ControlDirFormat.known_formats())
+        bzrdir.BzrDirFormat.unregister_lazy_format("Test format 1",
+            "bzrlib.tests.test_bzrdir", "BzrDirFormatTest1")
+        self.assertFalse(BzrDirFormatTest1 in
+            controldir.ControlDirFormat.known_formats())
+        self.assertRaises(KeyError, bzrdir.BzrProber.formats.get,
+            "Test format 1")
+
     def test_create_branch_and_repo_uses_default(self):
         format = SampleBzrDirFormat()
         branch = bzrdir.BzrDir.create_branch_and_repo(self.get_url(),

=== modified file 'bzrlib/tests/test_controldir.py'
--- bzrlib/tests/test_controldir.py	2011-02-24 16:23:16 +0000
+++ bzrlib/tests/test_controldir.py	2011-03-10 13:00:05 +0000
@@ -77,3 +77,30 @@
         formats = self.registry._get_all()
         self.assertEquals(1, len(formats))
         self.assertIsInstance(formats[0], SampleExtraComponentFormat)
+
+
+class ControlDirFormatTest1(controldir.ControlDirFormat):
+    """A test control dir format."""
+
+
+class TestControlDirFormat(tests.TestCaseWithTransport):
+    """Tests for the ControlDirFormat facility."""
+
+    def test_register_unregister_format(self):
+        format = ControlDirFormatTest1()
+        controldir.ControlDirFormat.register_format(format)
+        self.assertTrue(format in controldir.ControlDirFormat.known_formats())
+        controldir.ControlDirFormat.unregister_format(format)
+        self.assertFalse(format in controldir.ControlDirFormat.known_formats())
+
+    def test_register_unregister_format_lazy(self):
+        controldir.ControlDirFormat.register_lazy_format(
+            "bzrlib.tests.test_controldir", "ControlDirFormatTest1")
+        self.assertTrue(
+            ControlDirFormatTest1 in
+            controldir.ControlDirFormat.known_formats())
+        controldir.ControlDirFormat.unregister_lazy_format(
+            "bzrlib.tests.test_controldir", "ControlDirFormatTest1")
+        self.assertFalse(
+            ControlDirFormatTest1 in
+            controldir.ControlDirFormat.known_formats())

=== modified file 'bzrlib/tests/test_registry.py'
--- bzrlib/tests/test_registry.py	2011-02-24 16:26:25 +0000
+++ bzrlib/tests/test_registry.py	2011-03-10 12:59:19 +0000
@@ -352,3 +352,36 @@
         a_registry.register("obj", AThing())
         self.assertEquals("bzrlib.tests.test_registry",
             a_registry._get_module("obj"))
+
+
+class TestObjectGetter(tests.TestCase):
+
+    def test_eq(self):
+        obj = object()
+        objgetter1 = registry._ObjectGetter(obj)
+        objgetter2 = registry._ObjectGetter(obj)
+        self.assertEquals(objgetter1, objgetter1)
+        self.assertEquals(objgetter1, objgetter2)
+        self.assertNotEquals(objgetter1, registry._ObjectGetter(1))
+        self.assertNotEquals(objgetter1, 1)
+        self.assertEquals(objgetter1, obj)
+
+
+class TestLazyObjectGetter(tests.TestCase):
+
+    def test_eq_lazy(self):
+        # Use nonexisting objects to make sure all evaluation is lazy
+        objgetter1 = registry._LazyObjectGetter("nonexisting", "object")
+        objgetter2 = registry._LazyObjectGetter("nonexisting", "object")
+        objgetter3 = registry._LazyObjectGetter("nonexisting", "different")
+        self.assertEquals(objgetter1, objgetter1)
+        self.assertEquals(objgetter1, objgetter2)
+        self.assertNotEquals(objgetter1, objgetter3)
+
+    def test_eq(self):
+        objgetter1 = registry._LazyObjectGetter("bzrlib.tests.test_registry", "TestLazyObjectGetter")
+        objgetter2 = registry._LazyObjectGetter("bzrlib.tests.test_registry", "TestLazyObjectGetter")
+        self.assertEquals(objgetter1, objgetter2)
+        self.assertNotEquals(objgetter1, registry._ObjectGetter(1))
+        self.assertNotEquals(objgetter1, 1)
+        self.assertEquals(objgetter1, TestLazyObjectGetter)

=== modified file 'doc/en/release-notes/bzr-2.4.txt'
--- doc/en/release-notes/bzr-2.4.txt	2011-03-09 01:33:12 +0000
+++ doc/en/release-notes/bzr-2.4.txt	2011-03-10 13:29:54 +0000
@@ -170,6 +170,10 @@
   indicates whether the components of the bzrdir can be upgraded
   independent of the ``BzrDir``. (Jelmer Vernooij)
 
+* ``BzrProber.register_format`` and ``BzrProber.unregister_format`` are
+  now deprecated in favour of the ``BzrProber.formats`` format registry.
+  (Jelmer Vernooij)
+
 * ``ControlDir`` implementations no longer have to provide the ``get_branch_transport``,
   ``get_workingtree_transport`` and ``get_repository_transport`` methods.
   (Jelmer Vernooij, #730325)

